package com.cn.tianji.common;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.EnumUtils;

import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

@Slf4j
public class CommonUtils {
    /**
     * 计算时间差
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @return 秒
     */
    public static long timeDifference(Date startTime, Date endTime) {
        return timeDifference_(startTime,endTime) / 1000;
    }

    /**
     * 计算时间差
     * @param startTime
     * @param endTime
     * @return 毫秒
     */
    public static long timeDifference_(Date startTime, Date endTime) {
        return (endTime.getTime() - startTime.getTime()) ;
    }


    public static BigDecimal calculateAverage(List<BigDecimal> numbers) {
        if (numbers == null || numbers.isEmpty()) {
            throw new IllegalArgumentException("List of numbers cannot be null or empty.");
        }

        // 使用Stream API计算总和和元素数量
        BigDecimal sum = numbers.stream()
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        long count = numbers.size();

        // 计算平均值
        return sum.divide(new BigDecimal(count), 2, RoundingMode.HALF_UP);
    }

    /**
     * 计算数组最大值最小值差值
     * @param list
     * @return
     */
    public static  BigDecimal calculateDifference(List<BigDecimal> list){
        Optional<BigDecimal> maxOptional = list.stream().max(BigDecimal::compareTo);
        Optional<BigDecimal> minOptional = list.stream().min(BigDecimal::compareTo);
        BigDecimal difference=BigDecimal.ZERO;
        if (maxOptional.isPresent() && minOptional.isPresent()) {
            BigDecimal max = maxOptional.get();
            BigDecimal min = minOptional.get();
             difference = max.subtract(min);
//            System.out.println("最大值: " + max);
//            System.out.println("最小值: " + min);
//            System.out.println("差值: " + difference);
        } else {
            System.out.println("列表为空或无法计算最大值/最小值");
        }
        return difference;
    }
    /**
     * 去除时间字符串中的毫秒部分
     *
     * @param timeString 包含毫秒的时间字符串
     * @return 去除毫秒后的时间字符串
     */
    public static String removeMilliseconds(String timeString) {
        // 查找毫秒部分的起始位置（小数点）
        int dotIndex = timeString.indexOf('.');

        // 如果没有找到小数点，说明没有毫秒部分，直接返回原字符串
        if (dotIndex == -1) {
            return timeString;
        }

        // 毫秒部分的起始位置是小数点之后，所以返回小数点之前的子字符串
        return timeString.substring(0, dotIndex);
    }

    /**
     * 井表名转化井号
     * @param jhTableName 井表名
     * @return 井号
     */
    public static String getJh(String jhTableName){
        String s = jhTableName.replace('_', '-').toUpperCase();
        return s.replace("LTD-", "");
    }

    /**
     * 井号转化井表名
     * @param jh 井号
     * @return 井表名
     */
    public static String getJhTableName(String jh){
        String s = jh.replace('-', '_').toUpperCase();
        return "LTD_" + s ;
    }


    /**
     * 计算指定时间前/后时间
     * @param startDate 开始时间
     * @param type Calendar.HOUR/HOUR_OF_DAY/MINUTE/SECOND
     * @param timeDiff
     * @return
     */
    public static Date getSpecifiedTime(Date startDate,Integer type, Integer timeDiff){
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startDate); // 设置Calendar的时间为指定时间
//        Calendar.SECOND
        // 在Calendar中减去10秒
        calendar.add(type, timeDiff);
        // 获取调整后的时间
        return calendar.getTime();
    }

    /**
     * 判断某个枚举是否包某个value值
     * @param enumClass 需要判断是否存在那个枚举类中
     * @param action 需要判断的值
     * @return 包含返回true，否则返回false
     */
    public static boolean  diagnosisWarnAction(Class enumClass, String action){
        List enumList = EnumUtils.getEnumList(enumClass);
        for (int i = 0;i<enumList.size(); i++){
            Object en = enumList.get(i);
            Class<?> enClass = en.getClass();
            try {
                // 需要与枚举类方法对应
                Method method = enClass.getMethod("getValue");
                Object invoke = method.invoke(en, null);
                if(Objects.equals(invoke.toString(), action)) {
                    return true;
                }
            }catch (Exception e){
                log.error("枚举执行getCode方法失败...");
            }
        }
        return false;
    }

}
