package com.cn.tianji.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.conditions.update.LambdaUpdateChainWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cn.tianji.entity.PumpingPressureWarnLog;
import com.cn.tianji.mapper.PumpingPressureWarnLogMapper;
import com.cn.tianji.service.PumpingPressureWarnLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 泵压预警数据;(pumping_pressure_warn_log)表服务实现类
 * @author : http://www.chiner.pro
 * @date : 2024-6-6
 */
@Service
public class PumpingPressureWarnLogServiceImpl  extends ServiceImpl<PumpingPressureWarnLogMapper, PumpingPressureWarnLog> implements PumpingPressureWarnLogService {

    @Autowired
    private PumpingPressureWarnLogMapper pumpingPressureWarnLogMapper;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    /**
     * 通过ID查询单条数据
     *
     * @param jh 主键
     * @return 实例对象
     */
    @Override
    public PumpingPressureWarnLog queryById(String jh){
        return pumpingPressureWarnLogMapper.selectById(jh);
    }

    /**
     * 新增数据
     *
     * @param pumpingPressureWarnLog 实例对象
     * @return 实例对象
     */
    @Override
    public PumpingPressureWarnLog insert(PumpingPressureWarnLog pumpingPressureWarnLog){
        pumpingPressureWarnLogMapper.insert(pumpingPressureWarnLog);
        return pumpingPressureWarnLog;
    }

    /**
     * 更新数据
     *
     * @param pumpingPressureWarnLog 实例对象
     * @return 实例对象
     */
    @Override
    public PumpingPressureWarnLog update(PumpingPressureWarnLog pumpingPressureWarnLog){
        //1. 根据条件动态更新
        LambdaUpdateChainWrapper<PumpingPressureWarnLog> chainWrapper = new LambdaUpdateChainWrapper<PumpingPressureWarnLog>(pumpingPressureWarnLogMapper);
        if(StrUtil.isNotBlank(pumpingPressureWarnLog.getId())){
            chainWrapper.eq(PumpingPressureWarnLog::getId, pumpingPressureWarnLog.getId());
        }
        if(StrUtil.isNotBlank(pumpingPressureWarnLog.getMotion())){
            chainWrapper.eq(PumpingPressureWarnLog::getMotion, pumpingPressureWarnLog.getMotion());
        }
        if(StrUtil.isNotBlank(pumpingPressureWarnLog.getWorkingCondition())){
            chainWrapper.eq(PumpingPressureWarnLog::getWorkingCondition, pumpingPressureWarnLog.getWorkingCondition());
        }
        //2. 设置主键，并更新
        chainWrapper.set(PumpingPressureWarnLog::getJh, pumpingPressureWarnLog.getJh());
        boolean ret = chainWrapper.update();
        //3. 更新成功了，查询最最对象返回
        if(ret){
            return queryById(pumpingPressureWarnLog.getJh());
        }else{
            return pumpingPressureWarnLog;
        }
    }

    /**
     * 生成泵压预警日志数据实体类
     * @param jh 井号
     * @param workingCondition 工况
     * @param motion 动作
     * @param warnTime 预警点时间
     * @param warnSpp 预警点立管压力
     * @param warnTotalPump 预警点总泵冲
     * @param referenceTime 参考点时间
     * @param referenceSpp 参考点立管压力
     * @param referenceTotalPump 参考点总泵冲
     * @param totalPumpSub 立管压力差值
     * @param sppSub 立管压力差值
     * @param sppWarnValue 立管压力阈值
     * @param pumpWarnValue 总泵冲阈值
     * @return PumpingPressureWarnLog
     */
    @Override
    public PumpingPressureWarnLog createPumpingPressureWarnLog(String jh, String workingCondition , String motion, Date warnTime, BigDecimal warnSpp,BigDecimal warnTotalPump,
                                             Date referenceTime,BigDecimal referenceSpp,BigDecimal referenceTotalPump,BigDecimal totalPumpSub,BigDecimal sppSub,BigDecimal sppWarnValue,BigDecimal pumpWarnValue){
        PumpingPressureWarnLog model=new PumpingPressureWarnLog();
        String warnDes="符合泵压变化异常预警条件：<br>当前点立压:"+warnSpp+" Mpa,当前点时间:"+ sdf.format(warnTime)+";<br>参考点立压:"+referenceSpp+" Mpa,参考点时间:"+sdf.format(referenceTime)+";<br> 泵压差值:"+sppSub+" Mpa;满足阈值:"+sppWarnValue+"%)";
        model.setJh(jh);
        model.setWorkingCondition(workingCondition);
        model.setMotion(motion);
        model.setWarnTime(warnTime);
        model.setWarnSpp(warnSpp);
        model.setWarnTotalPump(warnTotalPump);
        model.setReferenceTime(referenceTime);
        model.setReferenceSpp(referenceSpp);
        model.setReferenceTotalPump(referenceTotalPump);
        model.setSppWarnValue(sppWarnValue);
        model.setPumpWarnValue(pumpWarnValue);
        model.setTotalPumpSub(totalPumpSub);
        model.setSppSub(sppSub);
        model.setWarnDes(warnDes);
        model.setCreateTime(new Date());
        return model;
    }
}
