package com.cn.tianji.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.conditions.update.LambdaUpdateChainWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cn.tianji.entity.TorqueWarnLog;
import com.cn.tianji.mapper.TorqueWarnLogMapper;
import com.cn.tianji.service.TorqueWarnLogService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 扭矩预警日志表;(torque_warn_log)表服务实现类
 * @author : http://www.chiner.pro
 * @date : 2024-6-13
 */
@Service
@Slf4j
public class TorqueWarnLogServiceImpl  extends ServiceImpl<TorqueWarnLogMapper, TorqueWarnLog> implements TorqueWarnLogService {

    @Resource
    private TorqueWarnLogMapper torqueWarnLogMapper;
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public TorqueWarnLog queryById(String id){
        return torqueWarnLogMapper.selectById(id);
    }

    /**
     * 新增数据
     *
     * @param torqueWarnLog 实例对象
     * @return 实例对象
     */
    @Override
    public TorqueWarnLog insert(TorqueWarnLog torqueWarnLog){
        torqueWarnLogMapper.insert(torqueWarnLog);
        return torqueWarnLog;
    }

    /**
     * 更新数据
     *
     * @param torqueWarnLog 实例对象
     * @return 实例对象
     */
    @Override
    public TorqueWarnLog update(TorqueWarnLog torqueWarnLog){
        //1. 根据条件动态更新
        LambdaUpdateChainWrapper<TorqueWarnLog> chainWrapper = new LambdaUpdateChainWrapper<TorqueWarnLog>(torqueWarnLogMapper);
        if(StrUtil.isNotBlank(torqueWarnLog.getJh())){
            chainWrapper.eq(TorqueWarnLog::getJh, torqueWarnLog.getJh());
        }
        if(StrUtil.isNotBlank(torqueWarnLog.getMotion())){
            chainWrapper.eq(TorqueWarnLog::getMotion, torqueWarnLog.getMotion());
        }
        if(StrUtil.isNotBlank(torqueWarnLog.getWorkingCondition())){
            chainWrapper.eq(TorqueWarnLog::getWorkingCondition, torqueWarnLog.getWorkingCondition());
        }
        //2. 设置主键，并更新
        chainWrapper.set(TorqueWarnLog::getId, torqueWarnLog.getId());
        boolean ret = chainWrapper.update();
        //3. 更新成功了，查询最最对象返回
        if(ret){
            return queryById(torqueWarnLog.getId());
        }else{
            return torqueWarnLog;
        }
    }

    /**
     * 生成扭矩预警日志数据实体类
     *
     * @param jh 井号
     * @param workingCondition 预警点工况
     * @param motion 预警点动作
     * @param warnStartTime 预警取值开始时间
     * @param warnEndTime 预警取值结束时间
     * @param warnTorque 预警点扭矩
     * @param warnMeanValue 预警点扭矩均值
     * @param warnFluctuatedValue 预警点扭矩波动值
     * @param warnBitdepth 预警点钻头位置
     * @param warnDepth 预警点井深
     * @param referenceStartTime 参考值起点时间
     * @param referenceEndTime 参考值结束时间
     * @param referenceMeanValue 参考值扭矩均值
     * @param referenceFluctuatedValue 参考值扭矩波动值
     * @param resMeanValue 均值计算结果值
     * @param resFluctuatedValue 波动值计算结果值
     * @param thresholdMeanValue 均值阈值
     * @param thresholdFluctuatedValue 波动值阈值
     * @return
     */
    @Override
    public TorqueWarnLog createTorqueWarnLog(String jh, String workingCondition , String motion, Date warnStartTime, Date warnEndTime, BigDecimal warnTorque,BigDecimal warnMeanValue,BigDecimal warnFluctuatedValue,
                                             BigDecimal warnBitdepth,BigDecimal warnDepth,Date referenceStartTime,Date referenceEndTime,BigDecimal referenceMeanValue,BigDecimal referenceFluctuatedValue, BigDecimal resMeanValue,
                                             BigDecimal resFluctuatedValue,BigDecimal thresholdMeanValue,BigDecimal thresholdFluctuatedValue,String isMeanWarn,String isFluctuatedWarn
                                             ){
        TorqueWarnLog model=new TorqueWarnLog();
        model.setJh(jh);
        model.setWorkingCondition(workingCondition);
        model.setMotion(motion);
        model.setWarnStartTime(warnStartTime);
        model.setWarnEndTime(warnEndTime);
        model.setWarnTorque(warnTorque);
        model.setWarnMeanValue(warnMeanValue);
        model.setWarnFluctuatedValue(warnFluctuatedValue);
        model.setWarnBitdepth(warnBitdepth);
        model.setWarnDepth(warnDepth);
        model.setReferenceStartTime(referenceStartTime);
        model.setReferenceEndTime(referenceEndTime);
        model.setReferenceMeanValue(referenceMeanValue);
        model.setReferenceFluctuatedValue(referenceFluctuatedValue);
        model.setResMeanValue(resMeanValue);
        model.setResFluctuatedValue(resFluctuatedValue);
        model.setThresholdMeanValue(thresholdMeanValue);
        model.setThresholdFluctuatedValue(thresholdFluctuatedValue);
        model.setCreateTime(new Date());
        model.setIsMeanWarn(isMeanWarn);
        model.setIsFluctuatedWarn(isFluctuatedWarn);
        return model;
    }

    /**
     * 新增扭矩预警日志
     * @param model
     */
    @Override
    public void saveTorqueWarnLog(TorqueWarnLog model){
        String warnDes="";
        String warnStart="符合扭矩变化异常预警条件";
        String warnContent="预警点扭矩:"+model.getWarnTorque().setScale(2, RoundingMode.HALF_UP)+"KN; <br>预警取值起点时间:"+sdf.format(model.getWarnStartTime())+
                "; 预警取值结束时间:"+sdf.format(model.getWarnEndTime())+"; <br>预警扭矩均值:"+model.getWarnMeanValue().setScale(2,RoundingMode.HALF_UP)+"KN; <br>预警扭矩波动值"+model.getWarnFluctuatedValue().setScale(2,RoundingMode.HALF_UP)+"KN; ";
        String warnType="";
        //参考值通用描述
        String referenceContent="(参考值)取值起点时间:"+sdf.format(model.getReferenceStartTime())+"; 取值结束时间:"+sdf.format(model.getReferenceEndTime());
        //均值参考值描述
        String referenceMeanContent="(变化量)扭矩均值:"+model.getReferenceMeanValue().setScale(2,RoundingMode.HALF_UP)+"KN;  均值计算结果值:"+model.getResMeanValue().multiply(BigDecimal.valueOf(100)).setScale(2,RoundingMode.HALF_UP)+"%; 均值阈值:"+model.getThresholdMeanValue().multiply(BigDecimal.valueOf(100)).setScale(2,RoundingMode.HALF_UP)+"%。";
        //拨动值参考值描述
        String referenceFluctuatedContent="(波动幅度)扭矩波动值:"+model.getReferenceFluctuatedValue().setScale(2,RoundingMode.HALF_UP)+"KN; 波动值计算结果值:"+model.getResFluctuatedValue().multiply(BigDecimal.valueOf(100)).setScale(2,RoundingMode.HALF_UP)+"%; 波动值阈值:"+model.getThresholdFluctuatedValue().multiply(BigDecimal.valueOf(100)).setScale(2,RoundingMode.HALF_UP)+"%。 ";
        String endContent="";
        if("1".equals(model.getIsMeanWarn())&&"1".equals(model.getIsFluctuatedWarn())){
            warnType="(变化量及波动异常)";
            warnDes= warnStart+warnType+"<br>"+warnContent+"<br>"+referenceContent+"<br>"+referenceMeanContent+"<br>"+referenceFluctuatedContent+"<br>"+endContent;
        }else{
            if("1".equals(model.getIsMeanWarn())){
                warnType="(变化量异常)";
                warnDes= warnStart+warnType+"<br>"+warnContent+"<br>"+referenceContent+"<br>"+referenceMeanContent+"<br>"+endContent;
            }else if("1".equals(model.getIsFluctuatedWarn())){
                warnType="(波动异常)";
                warnDes= warnStart+warnType+"<br>"+warnContent+"<br>"+referenceContent+"<br>"+referenceFluctuatedContent+"<br>"+endContent;
            }
        }
        log.info(warnDes);
        model.setWarnDes(warnDes);
        insert(model);
    }
}
