package com.cn.tianji.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.conditions.update.LambdaUpdateChainWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cn.tianji.entity.TotalPoolVolumeWarnLog;
import com.cn.tianji.mapper.TotalPoolVolumeWarnLogMapper;
import com.cn.tianji.service.TotalPoolVolumeWarnLogService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.xml.crypto.Data;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 总池体积预警数据日志表;(total_pool_volume_warn_log)表服务实现类
 * @author : zrj
 * @date : 2024-6-4
 */
@Slf4j
@Service
public class TotalPoolVolumeWarnLogServiceImpl  extends ServiceImpl<TotalPoolVolumeWarnLogMapper, TotalPoolVolumeWarnLog> implements TotalPoolVolumeWarnLogService {
    @Autowired
    private TotalPoolVolumeWarnLogMapper totalPoolVolumeWarnLogMapper;
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public TotalPoolVolumeWarnLog queryById(String id){
        return totalPoolVolumeWarnLogMapper.selectById(id);
    }

    /**
     * 新增数据
     *
     * @param totalPoolVolumeWarnLog 实例对象
     * @return 实例对象
     */
    @Override
    public TotalPoolVolumeWarnLog insert(TotalPoolVolumeWarnLog totalPoolVolumeWarnLog){
        totalPoolVolumeWarnLogMapper.insert(totalPoolVolumeWarnLog);
        return totalPoolVolumeWarnLog;
    }

    /**
     * 更新数据
     *
     * @param totalPoolVolumeWarnLog 实例对象
     * @return 实例对象
     */
    @Override
    public TotalPoolVolumeWarnLog update(TotalPoolVolumeWarnLog totalPoolVolumeWarnLog){
        //1. 根据条件动态更新
        LambdaUpdateChainWrapper<TotalPoolVolumeWarnLog> chainWrapper = new LambdaUpdateChainWrapper<TotalPoolVolumeWarnLog>(totalPoolVolumeWarnLogMapper);
        if(StrUtil.isNotBlank(totalPoolVolumeWarnLog.getJh())){
            chainWrapper.eq(TotalPoolVolumeWarnLog::getJh, totalPoolVolumeWarnLog.getJh());
        }
        if(StrUtil.isNotBlank(totalPoolVolumeWarnLog.getMotion())){
            chainWrapper.eq(TotalPoolVolumeWarnLog::getMotion, totalPoolVolumeWarnLog.getMotion());
        }
        if(StrUtil.isNotBlank(totalPoolVolumeWarnLog.getWorkingCondition())){
            chainWrapper.eq(TotalPoolVolumeWarnLog::getWorkingCondition, totalPoolVolumeWarnLog.getWorkingCondition());
        }
        if(StrUtil.isNotBlank(totalPoolVolumeWarnLog.getTotalWarnType())){
            chainWrapper.eq(TotalPoolVolumeWarnLog::getTotalWarnType, totalPoolVolumeWarnLog.getTotalWarnType());
        }
        //2. 设置主键，并更新
        chainWrapper.set(TotalPoolVolumeWarnLog::getId, totalPoolVolumeWarnLog.getId());
        boolean ret = chainWrapper.update();
        //3. 更新成功了，查询最最对象返回
        if(ret){
            return queryById(totalPoolVolumeWarnLog.getId());
        }else{
            return totalPoolVolumeWarnLog;
        }
    }

    /**
     * 生成总池体积预警日志数据实体类
     *
     * @param jh 井号
     * @param workingCondition 工况
     * @param motion 动作
     * @param warnType 预警类别
     * @param warnTime 预警点时间
     * @param warnPittotal 预警点总池体积
     * @param referenceValue 5分钟总池体积变化参考值
     * @param referenceStartTime  5分钟总池体积变化开始时间
     * @param referenceEndTime 5分钟总池体积变化结束时间
     * @param rateStartPittotal 30秒总池体积变化参考值
     * @param rateStartTime 30秒总池体积变化开始时间
     * @param rateEndTime 30秒总池体积变化结束时间
     * @param pittotalSub  5分钟总池体积变化与当前点总池体积差值
     * @param ratePittotalSub 30秒总池体积变化与当前点总池体积差值
     * @param referenceWarnValue 5分钟总池体积变化预警阈值
     * @param rateWarnValue 30秒总池体积变化预警阈值
     * @param sizeStr 大于/小于
     */
    @Override
    public TotalPoolVolumeWarnLog createTotalPoolVolumeWarnLog(String jh, String workingCondition , String motion, String warnType, Date warnTime, BigDecimal warnPittotal, BigDecimal referenceValue, Date referenceStartTime,
                                           Date referenceEndTime,BigDecimal rateStartPittotal,Date rateStartTime,Date rateEndTime,BigDecimal pittotalSub,BigDecimal ratePittotalSub,BigDecimal referenceWarnValue,
                                           BigDecimal rateWarnValue,String sizeStr){
        TotalPoolVolumeWarnLog model=new TotalPoolVolumeWarnLog();
        String warnDes="符合总池体积变化异常预警条件("+warnType+")"+"<br>"+"当前点时间:"+sdf.format(warnTime)+";当前点总池体积:"+warnPittotal.setScale(2, RoundingMode.HALF_UP)+"m³; "+
                "<br>与参考平均值:"+referenceValue.setScale(2, RoundingMode.HALF_UP)+"m³ ;"+"<br>"+"(参考值开始时间:"+sdf.format(referenceStartTime)+";参考值结束时间:"+sdf.format(referenceEndTime)+"); 计算差值:"+pittotalSub+"m³;"+sizeStr+"预警值:"+referenceWarnValue+"m³ ; "+
                "<br>且与(30秒内)参考波动值:"+rateStartPittotal.setScale(2, RoundingMode.HALF_UP)+"m³ ;<br>(参考值开始时间 : "+sdf.format(rateStartTime)+" ; 参考值结束时间 : "+sdf.format(rateEndTime)+"); 计算差值"+ratePittotalSub+"m³;"+sizeStr+"预警值:"+rateWarnValue+"m³";
        log.info(warnDes);
        model.setJh(jh);
        model.setWorkingCondition(workingCondition);
        model.setMotion(motion);
        model.setTotalWarnType(warnType);
        model.setWarnTime(warnTime);
        model.setWarnPittotal(warnPittotal.setScale(2, RoundingMode.HALF_UP));
        model.setReferenceValue(referenceValue.setScale(2, RoundingMode.HALF_UP));
        model.setReferenceStartTime(referenceStartTime);
        model.setReferenceEndTime(referenceEndTime);
        model.setRateStartPittotal(rateStartPittotal.setScale(2, RoundingMode.HALF_UP));
        model.setRateStartTime(rateStartTime);
        model.setRateEndTime(rateEndTime);
        model.setPittotalSub(pittotalSub);
        model.setRatePittotalSub(ratePittotalSub);
        model.setReferenceWarnValue(referenceWarnValue);
        model.setRateWarnValue(rateWarnValue);
        model.setWarnDes(warnDes);
        model.setCreateTime(new Date());
        return model;
    }
}
