package com.cn.tianji.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cn.tianji.common.StringUtils;
import com.cn.tianji.entity.YjRealTimeLog;
import com.cn.tianji.mapper.YjRealTimeLogMapper;
import com.cn.tianji.service.WellDrillingService;
import com.cn.tianji.service.YjDrillingDataSupplementationLogService;
import com.cn.tianji.service.YjRealTimeLogService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class YjRealTimeLogServiceImpl extends ServiceImpl<YjRealTimeLogMapper, YjRealTimeLog> implements YjRealTimeLogService {

    @Resource
    private YjRealTimeLogMapper mapper;

    @Resource
    private WellDrillingService wellDrillingService;

    DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");


    @Override
    public void saveRealTimeLog(List<String> jhLit) {
        //清空表数据
        mapper.truncateTable();
        List<YjRealTimeLog> saveList = new ArrayList<>();
        jhLit.forEach(item -> {
            YjRealTimeLog saveModel = new YjRealTimeLog();
            saveModel.setJh(item);
            String jhTable = "LTD_" + item.replace('-', '_').toUpperCase().trim();
            String selectJh = "\"" + jhTable + "\"";
            String lastDate = wellDrillingService.getLastDateByJh(selectJh);
            if (!StringUtils.isEmpty(lastDate)) {
                lastDate = lastDate.contains(".") ? lastDate.substring(0, lastDate.indexOf(".")) : lastDate;
                Date lastDateTime = null;
                try {
                    lastDateTime = sdf.parse(lastDate);
                } catch (ParseException e) {
                    e.printStackTrace();
                }
                Date nowTime = new Date();
                saveModel.setRealTime(lastDateTime);
                saveModel.setCreateTime(nowTime);
                long l = calculateMinutesDifference(lastDateTime, nowTime);
                if (l > 30) {
                    saveModel.setStatus("离线");
                } else {
                    saveModel.setStatus("在线");
                }
            }
            saveList.add(saveModel);
        });
        this.saveBatch(saveList);
    }

    /**
     * 计算两个LocalDateTime对象之间的分钟数差异
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return 小时数差异
     */
    public static long calculateMinutesDifference(Date startTime, Date endTime) {
        long duration = endTime.getTime()-startTime.getTime();
        long minutes = (duration % (1000 * 60 * 60)) / (1000 * 60);
        return minutes; // 返回总小时数差异
    }


    /**
     * 批量更新实时井状态
     */
    @Override
    public void updateBatchJhStatus() {
        List<YjRealTimeLog> list = this.list();
        if (list.size() > 0) {
            list.forEach(item -> {
                String jh = item.getJh();
                String jhTable = "LTD_" + jh.replace('-', '_').toUpperCase().trim();
                String selectJh = "\"" + jhTable + "\"";
                String lastDate = wellDrillingService.getLastDateByJh(selectJh);
                if(!StringUtils.isEmpty(lastDate)){
                    Date lastTime = null;
                    try {
                        lastTime = sdf.parse(lastDate);
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }
                    item.setRealTime(lastTime);
                    Date nowTime = new Date();
                    long l = calculateMinutesDifference(lastTime, nowTime);
                    if (l > 10) {
                        item.setStatus("离线");
                    } else {
                        item.setStatus("在线");
                    }
                }
            });
            this.updateBatchById(list);
        }
    }
}
