package com.cn.tianji.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cn.tianji.dto.ConditionDeterminationDto;
import com.cn.tianji.entity.OperatingConditionThreshold;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.entity.WellDrillingSICP;
import com.cn.tianji.service.*;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * 数据补录逻辑层
 *
 */
@Service
@Slf4j
public class DataSupplementationServiceImpl implements DataSupplementationService {


    @Resource
    private PullDataSICPService pullDataSICPService;

    @Resource
    private WellDrillingService wellDrillingService;

    @Resource
    private OperatingConditionThresholdService operatingConditionThresholdService;

    @Resource
    private ConditionIdentificationService conditionIdentificationService;

    private static final SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");



    @Override
    @SneakyThrows
    public void sicpDataSupp(String jh, String statrTime, String endTime){
        String saveJh = "LTD_" + jh.replace('-', '_').toUpperCase().trim();
        Date currentStartTime = sdf.parse(statrTime);
        Date finalEndTime = sdf.parse(endTime);
        QueryWrapper<OperatingConditionThreshold> operatingConditionThresholdQueryWrapper = new QueryWrapper<>();
        operatingConditionThresholdQueryWrapper.lambda().eq(OperatingConditionThreshold::getJh, jh);
        OperatingConditionThreshold operatingConditionThresholdModel = operatingConditionThresholdService.getOne(operatingConditionThresholdQueryWrapper);
        boolean isEndPull = currentStartTime.before(finalEndTime);
        while (isEndPull){
            log.info("采集开始时间:{}",statrTime);
            List<WellDrillingSICP> sicpWellDataByJhTime = pullDataSICPService.getSicpWellDataByJhTime(jh, statrTime);
            log.info("数据长度:{}",sicpWellDataByJhTime.size());
            if(!sicpWellDataByJhTime.isEmpty()) {
                List<WellDrilling> saveList = wellDrillingService.sicpToLtdData(sicpWellDataByJhTime);
//                log.info("数据:{}",saveList);
                String time  = conditionDetermination(saveList, saveJh, jh, sdf.parse(statrTime), operatingConditionThresholdModel);
                //log.info("最后一位数据:{}",saveList.get(saveList.size() - 1));
                //String time = saveList.get(saveList.size() - 1).getTime();
//                log.info("最后时间：{}",time);
                Date parse = sdf.parse(time);
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(parse);
                calendar.add(Calendar.SECOND, 2);
                Date lastTime = calendar.getTime();
                statrTime=sdf.format(lastTime);
                isEndPull = lastTime.before(finalEndTime);
            }
        }
    }

    public String conditionDetermination(List<WellDrilling> saveList,String tableName, String jh,Date startTime,OperatingConditionThreshold operatingConditionThresholdModel){
        List<WellDrilling> startBeforData = getStartBeforData(tableName, startTime);
        String time="";
        for (int i = 0; i < saveList.size(); i++) {
            List<WellDrilling> calList = new ArrayList<>();
            List<WellDrilling> saveLists = new ArrayList<>();
            WellDrilling wellDrilling = saveList.get(i);
            log.info("识别点位数据时间:{}",wellDrilling.getTime());
            if(i<2){
                if(!startBeforData.isEmpty()){
                    calList.add(startBeforData.get(startBeforData.size()-2));
                    calList.add(startBeforData.get(startBeforData.size()-1));
                    startBeforData.remove(0);
                    startBeforData.add(wellDrilling);
                }
            }else{
                WellDrilling wellDrillingOne = saveList.get(i-2);
                WellDrilling wellDrillingTwo = saveList.get(i-1);
                calList.add(wellDrillingOne);
                calList.add(wellDrillingTwo);
            }

            calList.add(wellDrilling);
            if(calList.size()>=3){
//                log.info("识别的数据长度:{}",calList.size());
//                log.info("识别数据：{}",calList.get(calList.size()-1));
                ConditionDeterminationDto conditionDeterminationDto = conditionIdentificationService.conditionDetermination(calList, operatingConditionThresholdModel);
                wellDrilling.setMotion(conditionDeterminationDto.getMotion());
                wellDrilling.setOperatingMode(conditionDeterminationDto.getOperatingMode());
                saveLists.add(wellDrilling);
                wellDrillingService.insertTableBatch(tableName,saveLists);
            }
            time=wellDrilling.getTime();
        }
        log.info("识别结束最后的时间:{}",time);
        return time;
    }


    /**
     *
     * 获取数据库中采集数据之前的数据
     * @param tableName
     * @param endTime
     * @return
     */
    public  List<WellDrilling> getStartBeforData(String tableName, Date endTime){
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(endTime);
        calendar.add(Calendar.MINUTE, -2);
        Date startTime = calendar.getTime();
        String startTimeString = sdf.format(startTime);
        String endTimeString = sdf.format(endTime);
        QueryWrapper<WellDrilling> wrapper = new QueryWrapper<>();
        wrapper.apply("TO_DATE(SUBSTR(TIME, 1, 19), 'yyyy-MM-dd HH24:mi:ss') >= to_date({0},'yyyy-mm-dd hh24:mi:ss')", startTimeString);
        wrapper.apply("TO_DATE(SUBSTR(TIME, 1, 19), 'yyyy-MM-dd HH24:mi:ss') <= to_date({0},'yyyy-mm-dd hh24:mi:ss')", endTimeString);
        wrapper.lambda().orderByDesc(WellDrilling::getTime);
        return wellDrillingService.selectListOrcale(tableName,wrapper);
    }

}
