package com.cn.tianji.common.util;

import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;

import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantReadWriteLock;

public class CaffeineCacheUtil<K, V> {
    private final Cache<K, V> cache;

    private final ReentrantReadWriteLock lock = new ReentrantReadWriteLock();

    public CaffeineCacheUtil(long duration, TimeUnit timeUnit) {
        this.cache = Caffeine.newBuilder()
                .expireAfterWrite(duration, timeUnit)
                .build();
    }

    public V get(K key) {
        lock.readLock().lock();
        try {
            return cache.getIfPresent(key);
        } finally {
            lock.readLock().unlock();
        }
    }

    public void put(K key, V value) {
        lock.writeLock().lock();
        try {
            cache.put(key, value);
        } finally {
            lock.writeLock().unlock();
        }
    }

    public void invalidate(K key) {
        lock.writeLock().lock();
        try {
            cache.invalidate(key);
        } finally {
            lock.writeLock().unlock();
        }
    }

}
