package com.cn.tianji.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cn.tianji.common.RedisUtil;
import com.cn.tianji.common.util.CaffeineCacheUtil;
import com.cn.tianji.common.util.CommonUtils;
import com.cn.tianji.dto.ConditionDeterminationDto;
import com.cn.tianji.entity.*;
import com.cn.tianji.mapper.Report1Mapper;
import com.cn.tianji.mapper.Report3Mapper;
import com.cn.tianji.mapper.SyncRecordMapper;
import com.cn.tianji.mapper.WlwmroMapper;
import com.cn.tianji.service.*;
import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Service
@Slf4j
public class BizServiceImpl implements BizService {

    @Value("${biz.tj.appKey}")
    private String appKey;

    @Value("${biz.tj.appSecret}")
    private String appSecret;

    @Value("${biz.tj.url1}")
    private String url1;

    @Value("${biz.tj.url2}")
    private String url2;

    @Value("${biz.tj.secretKey}")
    private String secretKey;

    @Value("${biz.tj.dataUrl}")
    private String dataUrl;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    private String catchName = "_Operating";

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private RestTemplate restTemplate;

    @Resource
    private Report3Mapper report3Mapper;

    @Resource
    private Report1Mapper report1Mapper;

    @Resource
    private WellDrillingService wellDrillingService;

    @Resource
    private SyncRecordMapper syncRecordMapper;

    @Resource
    private ConditionIdentificationService conditionIdentificationService;

    @Resource
    private SendWellDrillingMessageService sendWellDrillingMessageService;

    @Resource
    private OperatingConditionThresholdService operatingConditionThresholdService;

    @Resource
    private WlwmroMapper wlwmroMapper;

    @Resource
    private AjhService ajhService;

    @Resource
    private YjRealTimeLogService yjRealTimeLogService;
    //@Scheduled(fixedDelay = 30L, timeUnit = TimeUnit.MINUTES)
    @Override
    public void syncTables() throws InterruptedException {
        List<String> jhListFromRemote = new ArrayList<>();
        List<String> jhListFormDb = new ArrayList<>();
        try {
            jhListFromRemote = getTablesFromRemote();
        } catch (Exception exception) {
            jhListFromRemote = getTablesFromDb();
        }
        jhListFormDb = getTablesFromDb();
        List<String> finalJhListFormDb = jhListFormDb;
        initRecordData();
        jhListFromRemote.forEach(jh -> {
            if (!finalJhListFormDb.contains(jh)) {
                this.report3Mapper.createTable(jh, System.currentTimeMillis());
                //创建该井表后新增一条该井工况识别参数数据
                saveOperating(jh);
                try {
                    Thread.sleep(5000L);
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                long dataCounts = this.report3Mapper.dataCounts(jh, null, null);
                if (dataCounts > 0L) {
                    String maxTime = this.report3Mapper.selectMaxTimeFromDblink(jh);
                    this.syncRecordMapper.insert((new SyncRecord()).setId(UUID.randomUUID().toString()).setTableName(jh).setCollectTime(maxTime));
                    this.report3Mapper.syncHistoryData(jh, maxTime);
                } else {
                    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                    String collectTime = formatter.format(LocalDateTime.now());
                    this.syncRecordMapper.insert((new SyncRecord()).setId(UUID.randomUUID().toString()).setTableName(jh).setCollectTime(collectTime));
                }
            }
        });
    }

    //    @Scheduled(fixedDelay = 30L, timeUnit = TimeUnit.SECONDS, initialDelay = 60L)
    public void syncData() {
        log.info("数据同步开始");
        long time = System.currentTimeMillis();
        List<SyncRecord> recordList = this.syncRecordMapper.selectList();
        try {
            recordList.parallelStream().forEach(record -> {
                String maxTime = this.report3Mapper.selectMaxTimeFromDblink(record.getTableName());
                if (!record.getCollectTime().equals(maxTime)) {
                    record.setStartTime(record.getCollectTime()).setEndTime(maxTime);
                    try {
                        long time3 = System.currentTimeMillis();
                        this.report3Mapper.syncCurrentData(record);
                        long time4 = System.currentTimeMillis();
                        log.info("{}-耗时：{}", record.getTableName(), Long.valueOf(time4 - time3));
                    } catch (Exception ex) {
                        this.report3Mapper.dropTable(record.getTableName());
                        this.syncRecordMapper.deleteById(record.getId());
                        log.info("删除表{}", record.getTableName());
                        return;
                    }
                    this.syncRecordMapper.updateTimeById(record.setCollectTime(maxTime));
                }
            });
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        long time2 = System.currentTimeMillis();
        log.info("数据同步结束，耗时：" + (time2 - time));
    }

    /**
     * 程序启动则自动获取一次token
     *
     * @return
     */
    public String getToken() {
        JSONObject result1 = restTemplate.getForObject(url1 + "?appKey={1}&appSecret={2}", JSONObject.class, appKey, appSecret);
        String token = result1.getString("token");
        Repository.token = token;
        return token;
    }

    public List<String> getTablesFromRemote() {
        if (Repository.token == null) {
            getToken();
        }
        List<String> list = new ArrayList<>();
        JSONObject result = (JSONObject) this.restTemplate.getForObject(this.url2 + "?token={1}", JSONObject.class, new Object[]{Repository.token});
        JSONArray data = result.getJSONArray("data");
        JSONArray jhJsonArray = data.getJSONObject(0).getJSONArray("datas");
        log.info("井数量{}", jhJsonArray.size());
        log.info("同步井数据{}", jhJsonArray);
        List<String> jhs = new ArrayList<>();
        list = (List<String>) jhJsonArray.stream().map(it -> {
            String jh = JSONObject.parseObject(JSON.toJSONString(it)).getString("jh");
            jhs.add(jh);
            jh = "LTD_" + jh.replace('-', '_').toUpperCase().trim();
            return "\"" + jh + "\"";
        }).collect(Collectors.toList());
        //更新井工况识别参数缓存
        updateOperatingConditionCatch(jhs);
        return list;
    }

    public List<String> getTablesFromRemoteNotSplit() {
        if (Repository.token == null)
            getToken();
        List<String> list = new ArrayList<>();
        JSONObject result = (JSONObject) this.restTemplate.getForObject(this.url2 + "?token={1}", JSONObject.class, new Object[]{Repository.token});
        JSONArray data = result.getJSONArray("data");
        JSONArray jhJsonArray = data.getJSONObject(0).getJSONArray("datas");
        List<String> jhs = new ArrayList<>();
        log.info("井数量{}", jhJsonArray.size());
        log.info("同步井数据{}", jhJsonArray);
        list = (List<String>) jhJsonArray.stream().map(it -> {
            String jh = JSONObject.parseObject(JSON.toJSONString(it)).getString("jh");
            jhs.add(jh);
            jh = jh.replace('-', '_').toUpperCase().trim();
            return jh;
        }).collect(Collectors.toList());
        updateOperatingConditionCatch(jhs);
        return list;
    }

    /**
     * 更新每口井工况识别参数
     *
     * @param jhs 井号列表
     */
    public void updateOperatingConditionCatch(List<String> jhs) {
        QueryWrapper<OperatingConditionThreshold> wrapper = new QueryWrapper<>();
        wrapper.lambda().in(OperatingConditionThreshold::getJh, jhs);
        List<OperatingConditionThreshold> list = operatingConditionThresholdService.list(wrapper);
        list.forEach(item -> {
            String jh = item.getJh();
            String redisKsy = jh + catchName;
            redisUtil.set(redisKsy, item);
        });
    }

    public List<String> getTablesFromDb() {
        List<String> jhList = this.report3Mapper.tableNames();
        jhList.remove("SYNC_RECORD");
        jhList.remove("WELL_MATH_TIME");
        jhList.remove("WELL_NEW_TABLE");
        jhList.remove("WELL_MATH_UP_TIME");
        jhList.remove("DRILLING_CONDITIONS_LOG");
        return (List<String>) jhList.stream().map(it -> "\"" + it + "\"").collect(Collectors.toList());
    }

    public void initRecordData() {
        List<String> tableNameList = getTablesFromDb();
        if (Repository.initRecord) {
            this.syncRecordMapper.truncate();
            Repository.initRecord = false;
            tableNameList.forEach(jh -> {
                String maxTime = this.report3Mapper.selectMaxTime(jh);
                if (maxTime == null) {
                    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                    maxTime = formatter.format(LocalDateTime.now());
                }
                this.syncRecordMapper.insert((new SyncRecord()).setId(UUID.randomUUID().toString()).setTableName(jh).setCollectTime(maxTime));
            });
        } else {
            List<String> recordList = (List<String>) this.syncRecordMapper.selectList().stream().map(record -> record.getTableName()).collect(Collectors.toList());
            tableNameList.forEach(jh -> {
                if (!recordList.contains(jh)) {
                    String maxTime = this.report3Mapper.selectMaxTime(jh);
                    if (maxTime == null) {
                        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                        maxTime = formatter.format(LocalDateTime.now());
                    }
                    this.syncRecordMapper.insert((new SyncRecord()).setId(UUID.randomUUID().toString()).setTableName(jh).setCollectTime(maxTime));
                }
            });
        }
        log.info("初始化sync_record表结束");
    }


    /**
     * 记录
     * 1. 对比newTable表没有则记录
     * 2. 同步之前数据更新newtable表
     * 3. 同步之后的数据
     */
    @Override
    public void taskNewTable() {
        List<String> list = this.getSyncTableList();
        List<String> newTables = report3Mapper.newTableNames();

        for (String jh : list) {
            if (!newTables.contains(jh)) {
                WellNewTable newTable = new WellNewTable();
                newTable.setTableName(jh);
                newTable.setStatus(StatusModel.syncWait);
                newTable.setLastTime("");
                report3Mapper.insertNewTables(newTable);
            }

        }
    }

    /**
     * 同步新数据
     */
    @Override
    public void syncTableDataAfter() {

        List<String> list = this.getSyncTableList();

        this.createTable(list);

        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        for (String jh : list) {
            String saveJh = "LTD_" + jh.replace('-', '_').toUpperCase().trim();
            String selectJh = "\"" + saveJh + "\"";
            String lastDate = report3Mapper.getLastDateByJh(selectJh);
            try {
                //今天0点
                LocalDateTime lastDateTime = LocalDateTime.of(LocalDate.now(), LocalTime.MIDNIGHT);
                if (!StringUtils.isEmpty(lastDate)) {
                    lastDate = lastDate.contains(".") ? lastDate.substring(0, lastDate.indexOf(".")) : lastDate;
                    LocalDateTime formatTime = LocalDateTime.parse(lastDate, df);
                    lastDateTime = formatTime.isAfter(lastDateTime) ? formatTime : lastDateTime;
                }
                LocalDateTime addDate = lastDateTime.plusSeconds(1L);
                LocalDateTime endDate = lastDateTime.plusHours(2);
                String startTime = addDate.format(df);
                String endTime = endDate.format(df);
                this.syncTableData(jh, startTime, endTime);

            } catch (Exception e) {
                System.out.println("时间格式出错,错误井号：" + selectJh + "错误时间:" + lastDate);
            }


        }

    }


    /**
     * 同步老数据(从后往前)
     */
    @Override
    public void syncTableDataBefore() {

        List<String> list = report3Mapper.selectNeedSyncTable();

        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        this.createTable(list);

        for (String jh : list) {
            WellNewTable newTable = new WellNewTable();
            newTable.setTableName(jh);
            String saveJh = "LTD_" + jh.replace('-', '_').toUpperCase().trim();
            String selectJh = "\"" + saveJh + "\"";
            String lastDate = report3Mapper.getFirstDateByJh(selectJh);
            if (!com.cn.tianji.common.StringUtils.isEmpty(lastDate)) {
                log.info("saveJh:{};lastDate:{}", saveJh, lastDate);
                if (lastDate.contains(".")) {
                    lastDate = lastDate.substring(0, lastDate.indexOf("."));
                }
                LocalDateTime lastDateTime = LocalDateTime.parse(lastDate, df);
                LocalDateTime addDate = lastDateTime.minusSeconds(1L);
                String endTime = addDate.format(df);
                String startTime = lastDateTime.minusHours(2).format(df);
                String requestString = this.getTableData(jh, startTime, endTime);
                newTable.setLastTime(startTime);
                newTable.setStatus(StatusModel.syncWait);

                if (requestString.equals("\"[]\"")) {
                    newTable.setStatus(StatusModel.syncSuccess);
                    report3Mapper.updateNewTable(newTable);
                    continue;
                }
                List<WellDrilling> startBeforeDate = getConditionStartList(jh, startTime);
                List<WellDrilling> recordList = this.formatRecord(requestString, jh);
                List<List<WellDrilling>> splitList = this.splitList(recordList, 10);
                int i = 0;
                List<WellDrilling> perData = new ArrayList<>();
                for (List<WellDrilling> records : splitList) {
                    if (i == 0) {
                        saveBatchData(selectJh, records, startBeforeDate);
                    } else {
                        List<WellDrilling> temp = new ArrayList<>();
                        temp.add(perData.get(perData.size() - 2));
                        temp.add(perData.get(perData.size() - 1));
                        saveBatchData(selectJh, records, temp);
                    }
                    perData = records;
                    i++;
                    //   report3Mapper.insertTableBatch(selectJh,records);
                }
                report3Mapper.updateNewTable(newTable);
            }

        }

    }


    public List<WellDrilling> formatRecord(String json, String jh) {
        //log.info("json:{}", json);

        try {
            json = json.replaceAll("\\\\", "");
            json = json.substring(1, json.length() - 1);
            List<WellDrilling> list = JSONObject.parseArray(json, WellDrilling.class);
            return list;
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("这里报错拉拉拉拉啊啦啦啦");
            log.error("井号:{};/n 异常数据:{};/n 异常信息:{}", jh, json, e.getMessage());
            return new ArrayList<>();
        }
    }

    /**
     * 按指定大小，分隔集合，将集合按规定个数分为n个部分
     *
     * @param <T>
     * @param list
     * @param len
     * @return
     */
    public <T> List<List<T>> splitList(List<T> list, int len) {

        if (list == null || list.isEmpty() || len < 1) {
            return Collections.emptyList();
        }

        List<List<T>> result = new ArrayList<>();

        int size = list.size();
        int count = (size + len - 1) / len;

        for (int i = 0; i < count; i++) {
            List<T> subList = list.subList(i * len, ((i + 1) * len > size ? size : len * (i + 1)));
            result.add(subList);
        }

        return result;
    }


    /**
     * 同步新数据
     *
     * @param jh
     * @param startTime
     * @param endTime
     */
    @Override
    public void syncTableData(String jh, String startTime, String endTime) {
        try {
            String url = String.format(dataUrl + "?jh=%s&begin=%s&end=%s&secretKey=%s", jh, startTime, endTime, secretKey);
            String saveJh = "LTD_" + jh.replace('-', '_').toUpperCase().trim();
            String selectJh = "\"" + saveJh + "\"";
            String requestString = restTemplate.getForObject(url, String.class);
            List<WellDrilling> list = this.formatRecord(requestString, jh);
            if (list.size() == 0) {
                return;
            }
            List<WellDrilling> startBeforeDate = getConditionStartList(jh, startTime);
            String maxTime = list.get(list.size() - 1).getTime();
            List<List<WellDrilling>> splitList = this.splitList(list, 10);
            int i = 0;
            List<WellDrilling> perData = new ArrayList<>();
            for (List<WellDrilling> records : splitList) {
                if (i == 0) {
                    saveBatchData(selectJh, records, startBeforeDate);
                } else {
                    List<WellDrilling> temp = new ArrayList<>();
                    temp.add(perData.get(perData.size() - 2));
                    temp.add(perData.get(perData.size() - 1));
                    saveBatchData(selectJh, records, temp);
                }
                perData = records;
                i++;
//                report3Mapper.insertTableBatch(selectJh,records);
            }
            report3Mapper.updateSyncRecordByJh(selectJh, maxTime);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }


    /**
     * 同步新数据
     *
     * @param jh
     * @param startTime
     * @param endTime
     */
    public String getTableData(String jh, String startTime, String endTime) {
        try {
            String url = String.format(dataUrl + "?jh=%s&begin=%s&end=%s&secretKey=%s", jh, startTime, endTime, secretKey);
            String requestString = restTemplate.getForObject(url, String.class);
            return requestString;

        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }

    private void createTable(List<String> list) {
        List<String> jhList = list.stream()
                .map(it -> {
                    String jh = it;
                    //jh="LTD_"+jh.replace('-', '_').replaceAll("\\(|\\)|\\（|\\）","").toUpperCase().trim();
                    jh = "LTD_" + jh.replace('-', '_').toUpperCase().trim();
                    return "\"" + jh + "\"";
                })
                .collect(Collectors.toList());
        //动态建表
        List<String> tableNames = report3Mapper.tableNames().stream().map(it -> "\"" + it + "\"").collect(Collectors.toList());
        //log.info("report3数据库中已存在的表:{}",tableNames.toString());
        List<String> newTableNames = jhList.stream()
                .filter(it -> !tableNames.contains(it))
                .collect(Collectors.toList());
        log.info("需要在report3数据库中动态创建的表:{}", newTableNames.toString());
        newTableNames.forEach(it -> {
            report3Mapper.createTable(it, System.currentTimeMillis());
            //创建该井表后新增一条该井工况识别参数数据
            saveOperating(it);
            String jh = getJh(it);
            ajhService.saveData(jh);
        });
    }

    @Override
    public List<String> getSyncTableList() {

//        JSONObject result1 = restTemplate.getForObject(url1 + "?appKey={1}&appSecret={2}", JSONObject.class, appKey, appSecret);
//        String token = result1.getString("token");
        String token = getToken();
        JSONObject result2 = restTemplate.getForObject(url2 + "?token={1}", JSONObject.class, token);
        JSONArray data = result2.getJSONArray("data");
        JSONArray jhJsonArray = data.getJSONObject(0).getJSONArray("datas");
        System.out.println(jhJsonArray.toJSONString());
        return jhJsonArray.stream()
                .map(it -> {
                    String jh = JSONObject.parseObject(JSON.toJSONString(it)).getString("jh");
                    return jh;
                })
                .collect(Collectors.toList());
    }



    public void setAjhLastDateTime() {

        List<String> jhList = report1Mapper.selectAllAjhList();
        System.out.println(jhList);
        for (String jh : jhList) {
            String saveJh = "LTD_" + jh.replace('-', '_').toUpperCase().trim();
            String selectJh = "\"" + saveJh + "\"";
            try {
                String lastDate = report3Mapper.getLastDateByJh(selectJh);
                if (!StringUtils.isEmpty(lastDate)) {
                    lastDate = lastDate.split(" ")[0];
                    System.out.println(lastDate);
                    report1Mapper.updateAjhLastDate(jh, lastDate);
                }

            } catch (Exception e) {
                System.out.println(selectJh);
                System.out.println(e);
            }

        }

    }


    @Override
    public void updateAjHLastDate() {

        List<String> updateList = this.getTablesFromRemoteNotSplit();
//        System.out.println(updateList);
        for (String jh : updateList) {
            try {
                LocalDate currentDate = LocalDate.now();
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                String formattedDate = currentDate.format(formatter);
                report1Mapper.updateAjhLastDate(jh, formattedDate);
            } catch (Exception e) {
                System.out.println(jh);
                System.out.println(e);
            }
        }
    }

    @Override
    public void updateAjhJs() {

        List<String> jhList = report1Mapper.selectAllAjhList();

        for (String jh : jhList) {
            try {

                Map<String, Object> map = wlwmroMapper.selectJs(jh);
                if (map == null) {
                    continue;
                }
                report1Mapper.updateAjhJs(jh, map.get("DSJS").toString().replaceAll(".00", ""), map.get("SJJS").toString().replaceAll(".00", ""));
            } catch (Exception e) {
                System.out.println(jh);
                System.out.println(e);
            }

        }
    }


    public void saveBatchData(String selectJh, List<WellDrilling> records, List<WellDrilling> startBeforeDate) {
        //对数进行升序排序
//        log.info("排序数据时间:{}", records.get(0).getTIME());
        List<WellDrilling> collect = records;
//                .stream()
//                .sorted(Comparator.comparing(entity -> {
//                    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
//                    return LocalDateTime.parse(CommonUtils.removeMilliseconds(entity.getTime()), formatter);
//                }))
//                .collect(Collectors.toList());
        collect.sort((a, b) -> a.getTime().compareTo(b.getTime()));
        for (int i = 0; i < collect.size(); i++) {
            List<WellDrilling> models = new ArrayList<>();
            if (startBeforeDate.size() == 2) {
                if (i == 0) {
                    models.add(startBeforeDate.get(0));
                    models.add(startBeforeDate.get(1));
                    models.add(collect.get(i));
                } else if (i == 1) {
                    models.add(startBeforeDate.get(1));
                    models.add(collect.get(0));
                    models.add(collect.get(1));
                }
            }
            if (i >= 2) {
                models.add(collect.get(i - 2));
                models.add(collect.get(i - 1));
                models.add(collect.get(i));
            }
            log.info("工况识别数据长度:{}", models.size());
            if (models.size() == 3) {
                List<WellDrilling> wellDrillings = models;
                //WellDrillinghandle(models);
                OperatingConditionThreshold operatingConditionThreshold = getOperatingCatchByJh(wellDrillings.get(0).getJh());
                ConditionDeterminationDto conditionDeterminationDto = conditionIdentificationService.conditionDetermination(wellDrillings, operatingConditionThreshold);
                collect.get(i).setOperatingMode(conditionDeterminationDto.getOperatingMode());
                collect.get(i).setMotion(conditionDeterminationDto.getMotion());
                sendWellDrillingMessageService.sendMessage(conditionDeterminationDto.getMotion(), collect.get(i), collect.get(i).getJh());
            } else {
                collect.get(i).setOperatingMode("未识别工况");
                collect.get(i).setMotion("未识别动作");
            }
        }
        int i = wellDrillingService.insertTableBatch(selectJh, collect);
        log.info("井号;{};插入数据数量:{}", selectJh, i);
        //report3Mapper.insertTableBatch(selectJh, collect);
    }

    /**
     * 钻井参数处理
     *
     * @return
     */
    @Override
    public List<WellDrilling> WellDrillinghandle(List<WellDrilling> records) {
        List<WellDrilling> list = new ArrayList<>();
        records.forEach(item -> {
            WellDrilling model = new WellDrilling();
            model = item;
            list.add(model);
        });
        //对数进行升序排序
        List<WellDrilling> res = list.stream()
                .sorted(Comparator.comparing(entity -> {
                    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                    return LocalDateTime.parse(CommonUtils.removeMilliseconds(entity.getTime()), formatter);
                }))
                .collect(Collectors.toList());
        return res;
    }

    /**
     * 设置动作识别阈值
     *
     * @return
     */
    @Override
    public OperatingConditionThreshold setCreating() {
        OperatingConditionThreshold operatingConditionThreshold = new OperatingConditionThreshold();
        operatingConditionThreshold.setJh("通用");
        operatingConditionThreshold.setApproachingSurface(BigDecimal.valueOf(30));
        operatingConditionThreshold.setWoh(BigDecimal.valueOf(210));
        operatingConditionThreshold.setOnBottom(BigDecimal.valueOf(5));
        operatingConditionThreshold.setIsRotationOff(BigDecimal.valueOf(5));
        operatingConditionThreshold.setBlockldle(BigDecimal.valueOf(0));
        operatingConditionThreshold.setNoFlow(BigDecimal.valueOf(15));
        operatingConditionThreshold.setPressureZero(BigDecimal.valueOf(5));
        operatingConditionThreshold.setBitMovingUp(BigDecimal.valueOf(0));
        operatingConditionThreshold.setBlockMovingUp(BigDecimal.valueOf(0));
        operatingConditionThreshold.setFarBottom(BigDecimal.valueOf(30));
        return operatingConditionThreshold;
    }

    public List<WellDrilling> getConditionStartList(String jh, String endTime) {
        Date parse = null;
        try {
            parse = sdf.parse(endTime);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(parse);
        calendar.add(Calendar.SECOND, -20);
        String startTime = sdf.format(calendar.getTime());
        String tableData = getTableData(jh, startTime, endTime);
        List<WellDrilling> formatRecordS = this.formatRecord(tableData, jh);
        List<WellDrilling> collect = formatRecordS.stream().filter(item -> !item.getTime().equals(endTime)).collect(Collectors.toList());
        List<WellDrilling> startBeforeDate = new ArrayList<>();
        if (collect.size() >= 2) {
            startBeforeDate.add(collect.get(collect.size() - 1));
            startBeforeDate.add(collect.get(collect.size() - 2));
        }
        return startBeforeDate;
    }

    /**
     * 新增井工况识别参数数据
     *
     * @param jhTableName
     */
    public void saveOperating(String jhTableName) {
        OperatingConditionThreshold model = setCreating();
        String jh = getJh(jhTableName);
        log.info("设置工况识别参数:井号:{}",jh);
        QueryWrapper<OperatingConditionThreshold> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(OperatingConditionThreshold::getJh, jh);
        List<OperatingConditionThreshold> list = operatingConditionThresholdService.list(wrapper);
        if (list.size() == 0) {
            model.setJh(jh);
            log.info("设置工况识别参数:{}",model);
            operatingConditionThresholdService.save(model);
            redisUtil.set(jh + catchName, model);
        }
    }

    public String getJh(String jhTableName) {
        //井号预处理
        jhTableName = jhTableName.replace("\"", "");
        String s = jhTableName.replace('_', '-').toUpperCase();
        return s.replace("LTD-", "");
    }

    public OperatingConditionThreshold getOperatingCatchByJh(String jh) {
        String redisKsy = jh + catchName;
        OperatingConditionThreshold catchData = new OperatingConditionThreshold();
        boolean isHaveCatch = redisUtil.hasKey(redisKsy);
        if (isHaveCatch) {
            catchData = JSONObject.toJavaObject(JSONObject.parseObject(JSON.toJSONString(redisUtil.get(redisKsy))), OperatingConditionThreshold.class);
        } else {
           // catchData = setCreating();
            getOp(jh);
        }
        return catchData;
    }

    @Override
    public void getOp(String jh){
        String redisKsy = jh + catchName;
        QueryWrapper<OperatingConditionThreshold> wrapper=new QueryWrapper<>();
        wrapper.lambda().eq(OperatingConditionThreshold::getJh,jh);
        List<OperatingConditionThreshold> list = operatingConditionThresholdService.list(wrapper);
        if(list.size()>0){
            OperatingConditionThreshold operatingConditionThreshold = list.get(0);
            redisUtil.del(redisKsy);
            redisUtil.set(redisKsy,operatingConditionThreshold);
        }else{
            OperatingConditionThreshold saveModel = setCreating();
            saveModel.setJh(jh);
            operatingConditionThresholdService.save(saveModel);
        }
    }

    //    @PostConstruct
    public void createJhOperatingCatch() {
        List<String> jhList = report3Mapper.tableNames();
        jhList.remove("SYNC_RECORD");
        jhList.remove("WELL_MATH_TIME");
        jhList.remove("WELL_NEW_TABLE");
        jhList.remove("WELL_MATH_UP_TIME");
        jhList.remove("DRILLING_CONDITIONS_LOG");
        jhList.forEach(this::saveOperating);
    }

    CaffeineCacheUtil<String, String> cache = new CaffeineCacheUtil<>(48, TimeUnit.HOURS);

    //更新token方法 程序启动时执行一次 然后每天8点更新一次
    @Override
    public String getToken_caffeine() {
        JSONObject result1 = restTemplate.getForObject(url1 + "?appKey={1}&appSecret={2}", JSONObject.class, appKey, appSecret);
        String token = result1.getString("token");
        return token;
    }

    //更新数据采集井方法,程序启动执行一次 然后每隔8小时更新一次
    @Override
    public JSONArray pullJhList(String token) {
        JSONObject result2 = restTemplate.getForObject(url2 + "?token={1}", JSONObject.class, token);
        JSONArray data = result2.getJSONArray("data");
        JSONArray jhJsonArray = data.getJSONObject(0).getJSONArray("datas");
        cache.put("jhList", String.valueOf(jhJsonArray));
        List<String> jhList = jhJsonArray.stream()
                .map(it -> {
                    String jh = JSONObject.parseObject(JSON.toJSONString(it)).getString("jh");
                    return jh;
                })
                .collect(Collectors.toList());
        //更新创建井表
        this.createTable(jhList);
        jhList.add("顺北806斜");
        //更新实时数据表
        yjRealTimeLogService.saveRealTimeLog(jhList);
        return jhJsonArray;
    }

    @Override
    public  List<String>  getJhList() {
        String jhListStr = cache.get("jhList");
        JSONArray jhJsonArray = new JSONArray();
        if (jhListStr != null) {
            jhJsonArray = JSONArray.parseArray(jhListStr);
        } else {
            String token_caffeine = getToken_caffeine();
            jhJsonArray = pullJhList(token_caffeine);
        }
        List<String> jhList = jhJsonArray.stream()
                .map(it -> {
                    String jh = JSONObject.parseObject(JSON.toJSONString(it)).getString("jh");
                    return jh;
                })
                .collect(Collectors.toList());
        return jhList;
    }

    //实时数据采集方法,每20秒执行一次
    @Override
    public void pullRealTimeData(List<String> list) {
//        List<String> list = this.getJhList();

        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        for (String jh : list) {

            String saveJh = "LTD_" + jh.replace('-', '_').toUpperCase().trim();
            String selectJh = "\"" + saveJh + "\"";
            String lastDate = report3Mapper.getLastDateByJh(selectJh);
            log.info("采集井号:{}",jh);
            try {

                //今天0点
                LocalDateTime lastDateTime = LocalDateTime.of(LocalDate.now(), LocalTime.MIDNIGHT);
                if (!StringUtils.isEmpty(lastDate)) {
                    lastDate = lastDate.contains(".") ? lastDate.substring(0, lastDate.indexOf(".")) : lastDate;
                    LocalDateTime formatTime = LocalDateTime.parse(lastDate, df);
                    lastDateTime = formatTime.isAfter(lastDateTime) ? formatTime : lastDateTime;
                }
                LocalDateTime addDate = lastDateTime.plusSeconds(1L);
                LocalDateTime endDate = lastDateTime.plusHours(2);
                String startTime = addDate.format(df);
                String endTime = endDate.format(df);
                this.syncTableData(jh, startTime, endTime);

            } catch (Exception e) {
                System.out.println("时间格式出错,错误井号：" + selectJh + "错误时间:" + lastDate);
            }

        }
    }
}










