package com.cn.tianji.task;

import com.alibaba.fastjson.JSONArray;
import com.cn.tianji.common.CommonUtils;
import com.cn.tianji.service.BizService;
import com.cn.tianji.service.YjRealTimeLogService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * 定时任务调度
 *
 * @author Zhao RenJi
 */
@Component("PullDataTask")
@Slf4j
public class PullDataTask {

    @Resource
    private BizService bizService;

    @Resource
    private YjRealTimeLogService yjRealTimeLogService;

//    @Scheduled(fixedDelay = 30L, timeUnit = TimeUnit.MINUTES, initialDelay = 60L)
    public void syncTables() throws InterruptedException{
        bizService.syncTables();
    }

    /**
     * 记录
     * 1. 对比newTable表没有则记录
     * 2. 同步之前数据更新newtable表
     * 3. 同步之后的数据
     */
//    @Scheduled(fixedDelay = 20,timeUnit = TimeUnit.SECONDS,initialDelay = 30L)
    public void taskNewTable() {
        bizService.taskNewTable();
    }

    /**
     * 同步新数据
     */
//    @Scheduled(fixedDelay = 10,timeUnit = TimeUnit.SECONDS,initialDelay = 30L)
    public void syncTableDataAfter() {
        bizService.syncTableDataAfter();
    }

    /**
     * 同步老数据(从后往前)
     */
//    @Scheduled(fixedDelay = 30,timeUnit = TimeUnit.SECONDS,initialDelay = 30L )
    public void syncTableDataBefore(){
        bizService.syncTableDataBefore();
    }

    @Scheduled(cron = "0 35 8 * * ?")
    public void updateAjHLastDate(){
        bizService.updateAjHLastDate();
    }

    /**
     * 更新AJH的  实际井深/设计井深
     */
    @Scheduled(cron = "0 40 8 * * ?")
    public void updateAjhJs(){
        bizService.updateAjhJs();
    }

    /**
     * 数据初始化执行,程序启动执行一次，每天8点更新一次
     */
    @PostConstruct
    @Scheduled(cron = "0 30 8 * * ?")
    public void init(){
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime nowTime = LocalDateTime.now();
        log.info("执行时间:{}",nowTime.format(df));
        log.info("获取token和井号");
        String token_caffeine = bizService.getToken_caffeine();
        JSONArray jsonArray = bizService.pullJhList(token_caffeine);
        log.info("要采集井的数据:{}",jsonArray);
    }

    @PostConstruct
    @Scheduled(fixedDelay = 12,timeUnit = TimeUnit.HOURS,initialDelay = 30L)
    public void updateOp(){
        List<String> jhList = bizService.getJhList();
        jhList.forEach(item->{
            bizService.getOp(item);
        });
    }


    @Scheduled(fixedDelay = 30,timeUnit = TimeUnit.SECONDS,initialDelay = 30L)
    public void pullRealTimeData(){
        Date startTime = new Date();
        List<String> jhList = bizService.getJhList();
        log.info("缓存井列表数据:{}",jhList);
        if(jhList.size()>0){
            bizService.pullRealTimeData(jhList);
        }
        Date endTime = new Date();
        long warnTimeDiff = CommonUtils.timeDifference_(startTime, endTime);
        log.info("采集井实时数据开始时间:{};结束时间:{};耗时:{}毫秒",startTime,endTime,warnTimeDiff);
    }

    @Scheduled(fixedDelay = 30,timeUnit = TimeUnit.MINUTES,initialDelay = 30L)
    public void updateBatchJhStatusTask(){
        yjRealTimeLogService.updateBatchJhStatus();
    }

//    @Scheduled(fixedDelay = 15,timeUnit = TimeUnit.SECONDS,initialDelay = 30L)
//    public void test(){
//        List<String> jhList =new ArrayList<>();
//        jhList.add("利页6HF");
//        jhList.add("沙深1");
//        jhList.add("车页1HF");
//        bizService.pullRealTimeData(jhList);
//    }
}
