package com.cn.tianji.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.cn.tianji.common.RedisUtil;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.entity.YjPumpingWarnValue;
import com.cn.tianji.entity.YjTotalPoolWarnValue;
import com.cn.tianji.service.PumpingPressureService;
import com.cn.tianji.service.PumpingPressureWarnService;
import com.cn.tianji.service.YjPumpingWarnValueService;
import com.cn.tianji.util.CommonUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

/**
 * 钩载预警
 *
 * @author Zhao RenJi
 */
@Service
@Slf4j
public class PumpingPressureWarnServiceImpl implements PumpingPressureWarnService {

    @Resource
    private PumpingPressureService pumpingPressureService;

    @Resource
    private YjPumpingWarnValueService pumpingWarnValueService;

    @Resource
    private RedisUtil redisUtil;

    public String warnValueKey = "_pump_warn_value_key";

    @Override
    public void pumpingPressureWarn(WellDrilling nowPoint) {
        String jh = nowPoint.getJh();
        //当前点立压
        BigDecimal nowSpp = nowPoint.getSpp().setScale(2, RoundingMode.HALF_UP);
        String redisHasKey = jh + "_pumping_pressure_real_time_computing";
        List<YjPumpingWarnValue> warnValueDatas = getWarnValues(jh);
        YjPumpingWarnValue warnValueData = filterWarnData(nowSpp, warnValueDatas);
        BigDecimal pumpingWarnValue = warnValueData.getPumpingWarnValue();
        BigDecimal sppWarnValue = warnValueData.getSppWarnValue();
        BigDecimal warnBitdepth = warnValueData.getBitdepth();
        BigDecimal bitdepth = nowPoint.getBitdepth();
        int isIntoButtom = bitdepth.compareTo(warnBitdepth);
        if (isIntoButtom > 0) {
            pumpingPressureService.pumpingPressureCalculate(nowPoint, redisHasKey, pumpingWarnValue, sppWarnValue);
        } else {
            boolean b = redisUtil.hasKey(redisHasKey);
            if (b) {
                redisUtil.del(redisHasKey);
            }
        }
    }

    public List<YjPumpingWarnValue> getWarnValues(String jh) {
        String warnKey = jh + warnValueKey;
        boolean isWarnKey = redisUtil.hasKey(warnKey);
        List<YjPumpingWarnValue> list = new ArrayList<>();
        if (isWarnKey) {
            Object o = redisUtil.get(warnKey);
            list = CommonUtil.castList(o, YjPumpingWarnValue.class);
        } else {
            list = pumpingWarnValueService.getListByJh(jh);
            redisUtil.set(warnKey, list);
        }
        return list;
    }


    /**
     * @param nowSpp
     * @param warnValueDatas
     */
    public YjPumpingWarnValue filterWarnData(BigDecimal nowSpp, List<YjPumpingWarnValue> warnValueDatas) {
        AtomicReference<YjPumpingWarnValue> model = new AtomicReference<>(new YjPumpingWarnValue());
        warnValueDatas.forEach(item -> {
            BigDecimal pumpingMin = item.getPumpingMin();
            BigDecimal pumpingMax = item.getPumpingMax();
            Boolean minIsZero = pumpingMin.compareTo(BigDecimal.ZERO) <= 0;
            Boolean maxIsZero = pumpingMax.compareTo(BigDecimal.ZERO) > 0;
            boolean isMin = nowSpp.compareTo(pumpingMin) > 0;
            boolean isMax = nowSpp.compareTo(pumpingMax) <= 0;
            if (minIsZero) {
                model.set(warnValueDatas.get(0));
            } else {
                if (maxIsZero) {
                    if (isMin && isMax) {
                        model.set(item);
                    }
                } else {
                    if (isMin) {
                        model.set(item);
                    }
                }
            }
        });
        YjPumpingWarnValue yjPumpingWarnValue = model.get();
        return yjPumpingWarnValue;
    }


}
