package com.cn.tianji.rocketMq;

import com.alibaba.fastjson.JSONObject;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.service.TotalPoolVolumeWarnService;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyStatus;
import org.apache.rocketmq.common.message.MessageExt;
import org.apache.rocketmq.spring.annotation.ConsumeMode;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.PrintWriter;
import java.io.StringWriter;

@Slf4j
@Component
@RocketMQMessageListener(topic = "topic-well_log", consumerGroup = "${rocketmq.consumer.group}",
        selectorExpression = "*", consumeMode = ConsumeMode.ORDERLY, consumeThreadMax = 30)
public class SimpleConsumer implements RocketMQListener<MessageExt> {

    @Resource
    private TotalPoolVolumeWarnService totalPoolVolumeWarnService;

    @Override
    public void onMessage(MessageExt message) {
        // 打印传递的消息
        WellDrilling model = JSONObject.parseObject(message.getBody(), WellDrilling.class);
        try {
            log.info("Receive message 总池体积：线程名称:{},队列:{}，井号:{} ;时间:{};工况:{};动作:{},总池体积:{}", Thread.currentThread().getName(), message.getQueueId(), model.getJh(), model.getTime(), model.getOperatingMode(), model.getMotion(),model.getPittotal());
            totalPoolVolumeWarnService.totalPoolVolumeWarn(model);
        } catch (Exception e) {
            log.error("异常井：{},时间:{},工况:{},动作:{},总池体积:{}", model.getJh(), model.getTime(), model.getOperatingMode(), model.getMotion(),model.getPittotal());
            StringWriter sw= new StringWriter();
            PrintWriter pw= new PrintWriter(sw);
            e.printStackTrace(pw);
            String stackTrace=sw.toString();
            log.error(stackTrace);
        }
    }
}
