package com.cn.tianji.service;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author Zhao RenJi
 */
public interface ConditionDeterminationService {
    /**
     *
     * 判断是否:在井底
     * 井深-钻头深度<=在井底判定值
     * @param depth 井深
     * @param bitDepth 钻头深度
     * @param threshold 阈值
     * @return  true 在井底 false 未在井底
     */
    public Boolean isOnBottom(BigDecimal depth, BigDecimal bitDepth, BigDecimal threshold);

    /**
     * 判断是否:到达地面
     * 10s内钻头平均深度<在地面判定值
     * @param bitDepthList 10秒内钻头深度数组
     * @param threshold 阈值
     * @return true 到达地面 false 未到达地面
     */
    public Boolean isApproachingSurface(List<BigDecimal> bitDepthList, BigDecimal threshold);

    /**
     * 判断是否:离开井底
     * 10s内井深减钻头深度平均值>在地面判定值
     * @param bitDepthList 钻头深度数组
     * @param depthList     井深数组
     * @param threshold 阈值
     * @return true 离开井底 false 未离开井底
     */
    public Boolean isFarBottom(List<BigDecimal> bitDepthList,List<BigDecimal> depthList,BigDecimal threshold);

    /**
     * 判断是否:停转
     * 转速<停转判定值
     * @param rpm 转盘转速
     * @param threshold 阈值
     * @return true 停转 false 未停转
     */
    public Boolean isRotationOff(BigDecimal rpm,BigDecimal threshold);

    /**
     * 判断是否:停泵
     * 出口流量<出口流量设定值(泵冲)
     * @param pump1 一号泵冲
     * @param pump2 二号泵冲
     * @param pump3 三号泵冲
     * @param threshold 阈值
     * @return true 停泵 false 未停泵
     */
    public Boolean isNoFlow(BigDecimal pump1,BigDecimal pump2,BigDecimal pump3,BigDecimal threshold);

    /**
     * 判断是否:立压为0
     * 立压<立压设定值
     * @param spp 立管压力
     * @param threshold 阈值
     * @return true 立压为0 false 立压不为0
     */
    public Boolean isPressureZero(BigDecimal spp,BigDecimal threshold);

    /**
     * 判断是否:游车静止
     * 游车平均速度=0（大钩上升下降速度）
     * @param nowHkh 当前点大钩高度
     * @param perHkh 上一点大钩高度
     * @param timeSub 时间差
     * @param threshold 阈值
     * @return true 游车静止 false 游车未静止
     */
    public Boolean isBlockldle(BigDecimal nowHkh,BigDecimal perHkh,BigDecimal timeSub,BigDecimal threshold);

    /**
     * 判断是否:钻头上行
     * 钻头平均速度<钻头速度设定值
     * @param nowBitdepth 当前点钻头位置
     * @param perBitdepth 上一点钻头位置
     * @param timeSub 时间差
     * @param threshold 阈值
     * @return true 钻头上行 false 钻头下行
     */
    public Boolean isBitMovingUp(BigDecimal nowBitdepth,BigDecimal perBitdepth,BigDecimal timeSub,BigDecimal threshold);

    /**
     * 判断是否:游车上行
     * 游车平均速度>游车速度设定
     * @param nowHkh 当前点大钩高度
     * @param perHkh 上一点大钩高度
     * @param timeSub 时间差
     * @param threshold 阈值
     * @return true 游车上行 false 游车下行
     */
    public Boolean isBlockMovingUp(BigDecimal nowHkh,BigDecimal perHkh,BigDecimal timeSub,BigDecimal threshold);

    /**
     * 判断是否:坐卡
     * 钩载<钩载设定
     * @param wob 当前点钩载
     * @param threshold 阈值
     * @return true 坐卡 false 未坐卡
     */
    public Boolean isInSlips(BigDecimal wob,BigDecimal threshold);
}
