package com.cn.tianji.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cn.tianji.common.CommonUtils;
import com.cn.tianji.entity.*;
import com.cn.tianji.service.*;
import lombok.SneakyThrows;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 综合异常参数计算
 * @author Zhao RenJi
 */
@Service
public class ComprehensiveAlgorithmServiceImpl {

    @Resource
    private HookLoadWarnLogService hookLoadWarnLogService;

    @Resource
    private TorqueWarnLogService torqueWarnLogService;

    @Resource
    private TotalPoolVolumeWarnLogService totalPoolVolumeWarnLogService;

    @Resource
    private PumpingPressureWarnLogService pumpingPressureWarnLogService;

    @Resource
    private ComprehensiveLogService comprehensiveLogService;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    /**
     * 卡钻算法:
     * 起下钻：钩载 70% 总池体积 20%
     * 钻进:泵压 30% 扭矩 30% 钩载 20%
     */
    @SneakyThrows
    public void stickingOfTool(String jh, String time, String workingCondition){
        Date nowDateTime = sdf.parse(time);
        //获取当前时间30分钟前的时间
        Date startTime = CommonUtils.getSpecifiedTime(nowDateTime, Calendar.MINUTE, -30);
        if("起下钻".equals(workingCondition)){
            roundTripStickingOfTool( jh, startTime, nowDateTime,  workingCondition);
        }else{
            drillingStickingOfTool( jh, startTime, nowDateTime,  workingCondition);
        }
    }

    /**
     * 井漏溢流
     * @param jh
     * @param time
     * @param workingCondition
     */
    @SneakyThrows
    public void totalPoolVolume(String jh, String time, String workingCondition){
        Date nowDateTime = sdf.parse(time);
        //获取当前时间30分钟前的时间
        Date startTime = CommonUtils.getSpecifiedTime(nowDateTime, Calendar.MINUTE, -30);
        //井漏
        lostCirculation( jh, startTime, nowDateTime,  workingCondition);
        //溢流
        overflow(jh, startTime, nowDateTime,  workingCondition);
    }

    /**
     * 起下钻  卡钻
     */
    public void roundTripStickingOfTool(String jh,Date startTime,Date endTime, String workingCondition){
        ComprehensiveLog model =new ComprehensiveLog();
        model.setJh(jh);
        model.setWarnTime(sdf.format(endTime));
        model.setWorkingCondition(workingCondition);
        model.setWarnType("起下钻-卡钻");
        BigDecimal warnLevel = BigDecimal.ZERO;
        //查询钩载 总池体积预警数据 时间往前30分钟内
        QueryWrapper<HookLoadWarnLog> hookLoadWarnLogQueryWrapper =new QueryWrapper<>();
        hookLoadWarnLogQueryWrapper.lambda().eq(HookLoadWarnLog::getJh,jh).and(
                wrapper1 -> wrapper1
                        .apply("WARN_TIME >= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", startTime)
                        .apply("WARN_TIME <= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", endTime).or()
                        .apply("MOBIL_DRILLING_TOOLS_START_TIME >= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", startTime)
                        .apply("MOBIL_DRILLING_TOOLS_START_TIME <= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", endTime)
        );
        QueryWrapper<TotalPoolVolumeWarnLog> totalPoolVolumeWarnLogQueryWrapper=new QueryWrapper<>();
        totalPoolVolumeWarnLogQueryWrapper.lambda().eq(TotalPoolVolumeWarnLog::getJh,jh).and(
                wrapper1 -> wrapper1
                        .apply("WARN_TIME >= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", startTime)
                        .apply("WARN_TIME <= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", endTime)
        );
        List<HookLoadWarnLog> hookLoadWarnList = hookLoadWarnLogService.list(hookLoadWarnLogQueryWrapper);
        List<TotalPoolVolumeWarnLog> totalPoolVolumeWarnList = totalPoolVolumeWarnLogService.list(totalPoolVolumeWarnLogQueryWrapper);
        JSONObject warnIdsJson=new JSONObject();
        if(hookLoadWarnList.size()>0){
            warnLevel.add(BigDecimal.valueOf(70));
            List<String> hookLoadWarnIds = hookLoadWarnList.stream().map(HookLoadWarnLog::getId).collect(Collectors.toList());
            warnIdsJson.put("hookLoadWarnIds",hookLoadWarnIds);
        }
        if(totalPoolVolumeWarnList.size()>0){
            warnLevel.add(BigDecimal.valueOf(20));
            List<String> totalPoolVolumeWarnIds = totalPoolVolumeWarnList.stream().map(TotalPoolVolumeWarnLog::getId).collect(Collectors.toList());
            warnIdsJson.put("totalPoolVolumeWarnIds",totalPoolVolumeWarnIds);
        }
        model.setWarnLevel(warnLevel);
        model.setWarnIds(String.valueOf(warnIdsJson));
    }

    /**
     * 钻进 卡钻
     * @param jh
     * @param startTime
     * @param endTime
     * @param workingCondition
     */
    public void drillingStickingOfTool(String jh,Date startTime,Date endTime, String workingCondition){
        ComprehensiveLog model =new ComprehensiveLog();
        model.setJh(jh);
        model.setWarnTime(sdf.format(endTime));
        model.setWorkingCondition(workingCondition);
        model.setWarnType("钻进-卡钻");
        BigDecimal warnLevel = BigDecimal.ZERO;
        //查询钩载 扭矩 泵压预警数据 时间往前30分钟内
        QueryWrapper<HookLoadWarnLog> hookLoadWarnLogQueryWrapper =new QueryWrapper<>();
        hookLoadWarnLogQueryWrapper.lambda().eq(HookLoadWarnLog::getJh,jh).and(
                wrapper1 -> wrapper1
                        .apply("WARN_TIME >= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", startTime)
                        .apply("WARN_TIME <= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", endTime).or()
                        .apply("MOBIL_DRILLING_TOOLS_START_TIME >= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", startTime)
                        .apply("MOBIL_DRILLING_TOOLS_START_TIME <= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", endTime)
        );
        QueryWrapper<TorqueWarnLog> torqueWarnLogQueryWrapper =new QueryWrapper<>();
        torqueWarnLogQueryWrapper.lambda().eq(TorqueWarnLog::getJh,jh).and(
                wrapper1 -> wrapper1.
                        apply("WARN_END_TIME >= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", startTime)
                        .apply("WARN_END_TIME <= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", endTime)
        );

        QueryWrapper<PumpingPressureWarnLog> pumpingPressureWarnLogQueryWrapper =new QueryWrapper<>();
        pumpingPressureWarnLogQueryWrapper.lambda().eq(PumpingPressureWarnLog::getJh,jh).and(
                wrapper1 -> wrapper1.
                        apply("WARN_TIME >= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", startTime)
                        .apply("WARN_TIME <= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", endTime)
        );
        List<HookLoadWarnLog> hookLoadWarnList = hookLoadWarnLogService.list(hookLoadWarnLogQueryWrapper);
        List<TorqueWarnLog> torqueWarnList = torqueWarnLogService.list(torqueWarnLogQueryWrapper);
        List<PumpingPressureWarnLog> pumpingPressureWarnList = pumpingPressureWarnLogService.list(pumpingPressureWarnLogQueryWrapper);

        JSONObject warnIdsJson=new JSONObject();
        if(hookLoadWarnList.size()>0){
            warnLevel.add(BigDecimal.valueOf(20));
            List<String> hookLoadWarnIds = hookLoadWarnList.stream().map(HookLoadWarnLog::getId).collect(Collectors.toList());
            warnIdsJson.put("hookLoadWarnIds",hookLoadWarnIds);
        }
        if(torqueWarnList.size()>0){
            warnLevel.add(BigDecimal.valueOf(20));
            List<String> torqueWarnIds = torqueWarnList.stream().map(TorqueWarnLog::getId).collect(Collectors.toList());
            warnIdsJson.put("torqueWarnIds",torqueWarnIds);
        }
        if(pumpingPressureWarnList.size()>0){
            warnLevel.add(BigDecimal.valueOf(20));
            List<String> pumpingPressureWarnIds = pumpingPressureWarnList.stream().map(PumpingPressureWarnLog::getId).collect(Collectors.toList());
            warnIdsJson.put("pumpingPressureWarnIds",pumpingPressureWarnIds);
        }
        model.setWarnLevel(warnLevel);
        model.setWarnIds(String.valueOf(warnIdsJson));
    }
    /**
     * 井漏/溢流算法:
     * 钻进/起下钻：总池体积
     */
    public void lostCirculation(String jh,Date startTime,Date endTime, String workingCondition){
        ComprehensiveLog model =new ComprehensiveLog();
        model.setJh(jh);
        model.setWarnTime(sdf.format(endTime));
        model.setWorkingCondition(workingCondition);
        model.setWarnType("井漏");
        BigDecimal warnLevel = BigDecimal.ZERO;
        QueryWrapper<TotalPoolVolumeWarnLog> totalPoolVolumeWarnLogQueryWrapper=new QueryWrapper<>();
        totalPoolVolumeWarnLogQueryWrapper.lambda().eq(TotalPoolVolumeWarnLog::getJh,jh).and(
                wrapper1 -> wrapper1
                        .apply("WARN_TIME >= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", startTime)
                        .apply("WARN_TIME <= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", endTime)
        ).eq(TotalPoolVolumeWarnLog::getTotalWarnType,"井漏");
        List<TotalPoolVolumeWarnLog> totalPoolVolumeWarnList = totalPoolVolumeWarnLogService.list(totalPoolVolumeWarnLogQueryWrapper);
        JSONObject warnIdsJson=new JSONObject();
        if(totalPoolVolumeWarnList.size()>0){
            warnLevel.add(BigDecimal.valueOf(80));
            List<String> totalPoolVolumeWarnIds = totalPoolVolumeWarnList.stream().map(TotalPoolVolumeWarnLog::getId).collect(Collectors.toList());
            warnIdsJson.put("totalPoolVolumeWarnIds",totalPoolVolumeWarnIds);
        }
        model.setWarnLevel(warnLevel);
        model.setWarnIds(String.valueOf(warnIdsJson));
    }
    /**
     * 溢流算法:
     * 钻进/起下钻：总池体积
     */
    public void overflow(String jh,Date startTime,Date endTime, String workingCondition){
        ComprehensiveLog model =new ComprehensiveLog();
        model.setJh(jh);
        model.setWarnTime(sdf.format(endTime));
        model.setWorkingCondition(workingCondition);
        model.setWarnType("溢流");
        BigDecimal warnLevel = BigDecimal.ZERO;
        QueryWrapper<TotalPoolVolumeWarnLog> totalPoolVolumeWarnLogQueryWrapper=new QueryWrapper<>();
        totalPoolVolumeWarnLogQueryWrapper.lambda().eq(TotalPoolVolumeWarnLog::getJh,jh).and(
                wrapper1 -> wrapper1
                        .apply("WARN_TIME >= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", startTime)
                        .apply("WARN_TIME <= TO_DATE({0},'yyyy-MM-dd HH24:mi:ss')", endTime)
        ).eq(TotalPoolVolumeWarnLog::getTotalWarnType,"溢流");
        List<TotalPoolVolumeWarnLog> totalPoolVolumeWarnList = totalPoolVolumeWarnLogService.list(totalPoolVolumeWarnLogQueryWrapper);
        JSONObject warnIdsJson=new JSONObject();
        if(totalPoolVolumeWarnList.size()>0){
            warnLevel.add(BigDecimal.valueOf(80));
            List<String> totalPoolVolumeWarnIds = totalPoolVolumeWarnList.stream().map(TotalPoolVolumeWarnLog::getId).collect(Collectors.toList());
            warnIdsJson.put("totalPoolVolumeWarnIds",totalPoolVolumeWarnIds);
        }
        model.setWarnLevel(warnLevel);
        model.setWarnIds(String.valueOf(warnIdsJson));
    }
}
