package com.cn.tianji.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.cn.tianji.common.CommonUtils;
import com.cn.tianji.dto.ConditionDeterminationDto;
import com.cn.tianji.entity.OperatingConditionThreshold;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.service.ConditionIdentificationService;
import com.cn.tianji.service.DrillingConditionDeterminationService;
import com.cn.tianji.service.TrippingConditionDeterminationService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 工况识别
 *
 * @author Zhao RenJi
 */
@Service
@Slf4j
public class ConditionIdentificationServiceImpl implements ConditionIdentificationService {
    @Resource
    private ConditionDeterminationServiceImpl conditionDeterminationService;

    @Resource
    private TrippingConditionDeterminationService trippingConditionDeterminationService;

    @Resource
    private DrillingConditionDeterminationService drillingConditionDeterminationService;


    CommonUtils commonUtils = new CommonUtils();
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Override
    public ConditionDeterminationDto conditionDetermination(List<WellDrilling> wellDrillingLists, OperatingConditionThreshold operatingConditionThreshold) {
        ConditionDeterminationDto model = new ConditionDeterminationDto();

        //log.info("排序前的数据:{}",wellDrillingLists);
        List<WellDrilling> wellDrillingList = listReorder(wellDrillingLists);
//        log.info("排序后的数据长度:{}",wellDrillingList);
        String action = "未识别动作";
        WellDrilling nowPoint = wellDrillingList.get(wellDrillingList.size() - 1);
        WellDrilling perPoint = wellDrillingList.get(0);
        // log.info("工况识别开始:井号:{};识别点位时间:{};参考点位时间:{}",nowPoint.getJh(),nowPoint.getTime(),perPoint.getTime());
        nowPoint.setTime(CommonUtils.removeMilliseconds(nowPoint.getTime()));
        perPoint.setTime(CommonUtils.removeMilliseconds(perPoint.getTime()));
        model.setJh(nowPoint.getJh());
        model.setTime(nowPoint.getTime());
        try {
            long timeSub = 0;
            try {
                timeSub = commonUtils.timeDifference(sdf.parse(perPoint.getTime()), sdf.parse(nowPoint.getTime()));
            } catch (ParseException e) {
                e.printStackTrace();
            }
            BigDecimal timeSubBig = BigDecimal.valueOf(timeSub);
            List<BigDecimal> bitDepthList = wellDrillingList.stream().map(WellDrilling::getBitdepth).collect(Collectors.toList());
            List<BigDecimal> depthList = wellDrillingList.stream().map(WellDrilling::getDepth).collect(Collectors.toList());
            //离开井底阈值
            BigDecimal farBottomThresholdBig = operatingConditionThreshold.getFarBottom();
            //是否离开井底
            Boolean isFarBottom = conditionDeterminationService.isFarBottom(bitDepthList, depthList, farBottomThresholdBig);
            String workStatus = "起下钻";

            if (isFarBottom) {
                //起下钻工况
                action = trippingConditionDeterminationService.trippingConditionDetermination(wellDrillingList, timeSubBig, operatingConditionThreshold);
                workStatus = "起下钻";
            } else {
                //钻进工况
                action = drillingConditionDeterminationService.drillingConditionDetermination(nowPoint, perPoint, timeSubBig, operatingConditionThreshold);
                workStatus = "钻进";
            }

            model.setOperatingMode(workStatus);
            model.setMotion(action);
            log.info("工况识别结束：井号:{},当前点工况:{},当前点动作:{},当前点时间:{},参考点位时间:{}", nowPoint.getJh(), workStatus, action, nowPoint.getTime(), perPoint.getTime());

        } catch (Exception e) {
            log.info("工况识别异常:{}", e);
            model.setOperatingMode("未识别工况");
            model.setMotion("未识别动作");
        }
        return model;
    }

    /**
     * 对数组倒序排序
     *
     * @param list
     * @return
     */
    public List<WellDrilling> listReorder(List<WellDrilling> list) {
        // 使用Stream API进行排序
//        List<WellDrilling> sortedEntities = list.stream()
//                .sorted(Comparator.comparing(WellDrilling::getTime))
//                .collect(Collectors.toList());
        // 使用Comparator.comparing根据time属性进行倒序排序

        //  SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");

//        list.forEach(item->{
//            log.info("排序前数据:井号{};时间{}",item.getJh(),item.getTime());
//        });
        //正序
        list.sort((a, b) -> a.getTime().compareTo(b.getTime()));
        //倒序
        // list.sort((a, b) -> b.getTime().compareTo(a.getTime()));
//        Comparator<WellDrilling> dateTimeComparator  = Comparator.comparing(WellDrilling::getTime, (t1, t2) -> {
//            Date d1 = null;
//            Date d2 = null;
//            try {
//                d1 = sdf.parse(CommonUtils.removeMilliseconds(t1));
//                d2 = sdf.parse(CommonUtils.removeMilliseconds(t2));
//
//            } catch (ParseException e) {
//                e.printStackTrace();
//            }
//            return d2.compareTo(d1);
//        }).reversed();
//        list.sort(dateTimeComparator);
//        list.forEach(item->{
//            log.info("排序后数据:井号{};时间{}",item.getJh(),item.getTime());
//        });
        return list;
    }


}
