package com.cn.tianji.service.impl;

import com.cn.tianji.entity.OperatingConditionThreshold;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.enums.DrillingConditionDeterminationEnum;
import com.cn.tianji.service.CommonConditionDeterminationService;
import com.cn.tianji.service.DrillingConditionDeterminationService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;

@Service
public class DrillingConditionDeterminationServiceImpl implements DrillingConditionDeterminationService {

    @Resource
    private ConditionDeterminationServiceImpl conditionDeterminationService;

    @Resource
    private CommonConditionDeterminationService commonConditionDeterminationService;

    /**
     * 钻进工况动作判定
     * @param nowPoint 当前点数据
     * @param perPoint 上一点数据
     * @param timeSubBig 当前点与上一点时间差
     */
    @Override
    public String drillingConditionDetermination(WellDrilling nowPoint, WellDrilling perPoint, BigDecimal timeSubBig, OperatingConditionThreshold operatingConditionThreshold){
        String res= "";
        //坐卡阈值
        BigDecimal wobThresholdBig=operatingConditionThreshold.getWoh();
        BigDecimal nowWohBig = nowPoint.getWoh();
        Boolean isInSlips = conditionDeterminationService.isInSlips(nowWohBig, wobThresholdBig);
        //判断停泵及泵压为0
        Boolean publicPumpAndSpp = commonConditionDeterminationService.publicPumpAndSpp(nowPoint,operatingConditionThreshold.getNoFlow(),operatingConditionThreshold.getPressureZero());
        //判断是否停转
        Boolean publicRotationOff =commonConditionDeterminationService.publicRotationOff(nowPoint,operatingConditionThreshold.getIsRotationOff());
        if(isInSlips){
            res= DrillingConditionDeterminationEnum.InSlips.getValue();
        }else{
            BigDecimal depthBig = nowPoint.getDepth();
            BigDecimal bitdeptBig = nowPoint.getBitdepth();
            //在井底阈值
            BigDecimal onBottomThreshold = operatingConditionThreshold.getOnBottom();
            //判断是否在井底
            Boolean isOnBottom = conditionDeterminationService.isOnBottom(depthBig, bitdeptBig, onBottomThreshold);
            if(isOnBottom){
                //判断停泵及泵压为0
                if(publicPumpAndSpp){
                    //判断是否停转
                    if(publicRotationOff){
                        res= DrillingConditionDeterminationEnum.Standby.getValue();
                    }else{
                        res= DrillingConditionDeterminationEnum.DryDrill.getValue();
                    }
                }else{
                    //判断是否停转
                    if(publicRotationOff){
                        res= DrillingConditionDeterminationEnum.SlidingDrilling.getValue();
                    }else{
                        res= DrillingConditionDeterminationEnum.RotaryDrilling.getValue();
                    }
                }
            }else{
                res= publicJudgment( nowPoint,  perPoint,  timeSubBig,operatingConditionThreshold);
            }
        }
        return res;
    }

    /**
     *
     * @param nowPoint
     * @param perPoint
     * @param timeSubBig
     * @return
     */
    public String publicJudgment(WellDrilling nowPoint, WellDrilling perPoint, BigDecimal timeSubBig, OperatingConditionThreshold operatingConditionThreshold){
        String res= "";
        //判断游车是否静止
        Boolean isBlockldle = commonConditionDeterminationService.publicBlockldle(nowPoint, perPoint, timeSubBig,operatingConditionThreshold.getBlockldle());
        //判断是否停泵及立压为0
        Boolean isPumpAndSpp = commonConditionDeterminationService.publicPumpAndSpp(nowPoint,operatingConditionThreshold.getNoFlow(),operatingConditionThreshold.getPressureZero());
        //判断是否停转
        Boolean publicRotationOff = commonConditionDeterminationService.publicRotationOff(nowPoint,operatingConditionThreshold.getIsRotationOff());
        if(isBlockldle){
            //判断是否停泵及立压为0
            if(isPumpAndSpp){
                //判断是否停转
                if(publicRotationOff){
                    res= DrillingConditionDeterminationEnum.StopAndWait.getValue();
                }else{
                    res= DrillingConditionDeterminationEnum.StopPumpAndIdle.getValue();
                }
            }else{
                if(publicRotationOff){
                    res=DrillingConditionDeterminationEnum.StationaryCycle.getValue();
                }else{
                    res= DrillingConditionDeterminationEnum.CycleIdle.getValue();
                }
            }
        }else{
            //判断游车是否上行
            Boolean publicMovingUp = commonConditionDeterminationService.publicMovingUp(nowPoint, perPoint, timeSubBig,
                    operatingConditionThreshold.getBitMovingUp(),operatingConditionThreshold.getBlockMovingUp());
            if(publicMovingUp){
                //判断是否停泵及立压为0
                if(isPumpAndSpp){
                    //判断是否停转
                    if(publicRotationOff){
                        res= DrillingConditionDeterminationEnum.LiftingDrillingTools.getValue();
                    }else{
                        res= DrillingConditionDeterminationEnum.DryInvertedScratch.getValue();
                    }
                }else{
                    if(publicRotationOff){
                        res=DrillingConditionDeterminationEnum.LoopWhileLifting.getValue();
                    }else{
                        res= DrillingConditionDeterminationEnum.BackReaming.getValue();
                    }
                }
            }else {
                //判断是否停泵及立压为0
                if(isPumpAndSpp){
                    //判断是否停转
                    if(publicRotationOff){
                        res= DrillingConditionDeterminationEnum.LoweringDrillingTools.getValue();
                    }else{
                        res= DrillingConditionDeterminationEnum.DryEyeScratching.getValue();
                    }
                }else{
                    if(publicRotationOff){
                        res=DrillingConditionDeterminationEnum.LoopWhilePlacing.getValue();
                    }else{
                        res= DrillingConditionDeterminationEnum.Reaming.getValue();
                    }
                }
            }
        }
        return res;
    }
}
