package com.cn.tianji.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cn.tianji.common.CommonUtils;
import com.cn.tianji.common.DrillingOutHookLoadCatch;
import com.cn.tianji.common.RedisUtil;
import com.cn.tianji.entity.HookLoadCatch;
import com.cn.tianji.entity.HookLoadWarnLog;
import com.cn.tianji.entity.WarnHistory;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.enums.*;
import com.cn.tianji.service.DrillingOutHookLoadService;
import com.cn.tianji.service.HookLoadWarnLogService;
import com.cn.tianji.service.WarnHistoryService;
import com.cn.tianji.service.WellDrillingService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author Zhao RenJi
 */
@Slf4j
@Service
public class DrillingOutHookLoadServiceImpl implements DrillingOutHookLoadService {

    @Resource
    private WellDrillingService wellDrillingService;

    @Resource
    private WarnHistoryService warnHistoryService;

    @Resource
    private HookLoadWarnLogService hookLoadWarnLogService;

    @Resource
    private RedisUtil redisUtil;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Override
    public void hookLoad(String jhTableName, String startTime, String endTime) {
        QueryWrapper<WellDrilling> queryWrapper = new QueryWrapper<>();
        queryWrapper.apply("TO_DATE(SUBSTR(TIME, 1, 19), 'yyyy-MM-dd HH24:mi:ss') <= to_date({0},'yyyy-mm-dd hh24:mi:ss')", endTime);
        queryWrapper.apply("TO_DATE(SUBSTR(TIME, 1, 19), 'yyyy-MM-dd HH24:mi:ss') >= to_date({0},'yyyy-mm-dd hh24:mi:ss')", startTime);
        queryWrapper.orderByAsc("TIME");
        List<WellDrilling> list = wellDrillingService.selectListOrcale(jhTableName, queryWrapper);
        log.info("查询到的数据:{}", list);
    }

    /**
     * 钩载预警计算
     *
     * @param nowPoint 计算点位数据
     */
    @Override
    public void hookLoadWarnCalculate(WellDrilling nowPoint, String redisHasKey, BigDecimal warnValue, BigDecimal rodWeight,
                                      BigDecimal activityOne,BigDecimal activityTwo) {
        Date startTime = new Date();
        HookLoadCatch point = createPoint(nowPoint);
        //1.判断当前点工况
        //2.如果是上提下方钻具,则获取当前井是否由上一柱缓存数据
        //3.如果没有则记录当前点
        //4.当动作为坐卡时,判断当前点大钩位置是否位于最低位置,如果是则一柱记录结束,并根据一柱中钻头位置判断为起钻或者下钻
        //5.如果存在缓存数据,判断上一柱工况是否与当前点动作是否一致,如果一致,则进入预警判断
        //6.如果不一致,则判断是否存在当前柱缓存数据,如果不存在,默认为一柱起点,不记录与不判断
        //7.如果存在当前柱缓存数据,则判断是否存在当前点动作一致数据,
        String jh = nowPoint.getJh();
//        BigDecimal rodWeight = BigDecimal.valueOf(7.6);
//        BigDecimal warnValue = BigDecimal.valueOf(100);
//        String redisHasKey = jh + "_hook_load";
        String motion = nowPoint.getMotion();
        boolean isHave = redisUtil.hasKey(redisHasKey);
        boolean isWarnAction = CommonUtils.diagnosisWarnAction(TrippingConditionDeterminationWarnEnum.class, motion);
        //当前点是否为上提/下放钻具动作
        if (isWarnAction) {
            if (isHave) {
                //判断缓存数据是否超过当前数据30分钟
                boolean b = determineCachedDataTimeDiff(redisHasKey, point);
                if (b) {
                    redisUtil.del(redisHasKey);
                } else {
                    warnCalculation(redisHasKey, nowPoint, rodWeight, warnValue,activityOne,activityTwo);
                }
            } else {
                log.info("开始记录钩载数据");
                DrillingOutHookLoadCatch hookLoadCatch = new DrillingOutHookLoadCatch();
                hookLoadCatch.setJh(jh);
                hookLoadCatch.setRecordingSteps(1);
                List<HookLoadCatch> columnList = new ArrayList<>();
                columnList.add(point);
                hookLoadCatch.setPerColumnData(columnList);
                hookLoadCatch.setMobileDrillingToolsStep(0);
                hookLoadCatch.setIsMobileDrillingTools(false);
                hookLoadCatch.setNowColumnData(columnList);
                hookLoadCatch.setIfDetermineWorkingConditions(false);
                hookLoadCatch.setIsWarnNowPoint(false);
                hookLoadCatch = initializeMobileDrillingToolsData(hookLoadCatch);
                redisUtil.set(redisHasKey, hookLoadCatch);
            }
        } else {
            //如果动作不是上提/下方钻具
            //判断动作是否为坐卡
            if (isHave) {
                endCalculation(nowPoint, motion, redisHasKey);
            }
        }
        Date endTime = new Date();
        long l = CommonUtils.timeDifference_(startTime, endTime);
        log.info("井号:{},时间:{},工况:{},动作:{}, 计算耗时:{}", nowPoint.getJh(), nowPoint.getTime(), nowPoint.getOperatingMode(), nowPoint.getMotion(), l);
    }

    /**
     * 预警数据计算
     *
     * @param redisHasKey
     * @param nowPoint
     */
    public void warnCalculation(String redisHasKey, WellDrilling nowPoint, BigDecimal rodWeight,
                                BigDecimal warnValue,BigDecimal activityOne,BigDecimal activityTwo) {
        HookLoadCatch point = createPoint(nowPoint);
        DrillingOutHookLoadCatch hookLoadCatch = JSONObject.toJavaObject(JSONObject.parseObject(JSON.toJSONString(redisUtil.get(redisHasKey))), DrillingOutHookLoadCatch.class);
        Integer recordingSteps = hookLoadCatch.getRecordingSteps();
        if (recordingSteps == 1) {
            List<HookLoadCatch> perColumnData = hookLoadCatch.getPerColumnData();
            List<HookLoadCatch> nowColumnData = hookLoadCatch.getNowColumnData();
            perColumnData.add(point);
            nowColumnData.add(point);
            hookLoadCatch.setPerColumnData(perColumnData);
            hookLoadCatch.setNowColumnData(nowColumnData);
        } else if (recordingSteps == 2) {
            List<HookLoadCatch> nowColumnData = hookLoadCatch.getNowColumnData();
            if (nowColumnData == null) {
                nowColumnData = new ArrayList<>();
            }
            hookLoadCatch = continuousTrippingCalculationRules(nowPoint, hookLoadCatch, rodWeight, warnValue,activityOne,activityTwo);
            nowColumnData = hookLoadCatch.getNowColumnData();
            //当前点是否预警点
            if (!hookLoadCatch.getIsWarnNowPoint()) {
                nowColumnData.add(point);
            };
            hookLoadCatch.setNowColumnData(nowColumnData);
            hookLoadCatch.setIfDetermineWorkingConditions(false);
        }
        redisUtil.set(redisHasKey, hookLoadCatch);
    }


    /**
     * 判断缓存数据是否超出时间限制
     *
     * @param redisHasKey
     * @param nowPoint
     */
    @SneakyThrows
    public boolean determineCachedDataTimeDiff(String redisHasKey, HookLoadCatch nowPoint) {
        DrillingOutHookLoadCatch hookLoadCatch = JSONObject.toJavaObject(JSONObject.parseObject(JSON.toJSONString(redisUtil.get(redisHasKey))), DrillingOutHookLoadCatch.class);
        List<HookLoadCatch> perColumnData = hookLoadCatch.getPerColumnData();
        if(perColumnData.size()>0){
            HookLoadCatch catchData = perColumnData.get(perColumnData.size() - 1);
            String nowPointtime = nowPoint.getTime();
            String catchDatatime = catchData.getTime();
            long l = CommonUtils.timeDifference(sdf.parse(nowPointtime), sdf.parse(catchDatatime));
            int timeValue = 60 * 30;
            return l >= timeValue;
        }else{
            return false;
        }

    }

    /**
     * 非预警计算处理
     *
     * @param nowPoint
     * @param motion
     * @param redisHasKey
     */
    public void endCalculation(WellDrilling nowPoint, String motion, String redisHasKey) {
        if (motion.equals(TrippingConditionDeterminationEnum.InSlips.getValue())) {
            BigDecimal nowHkh = nowPoint.getHkh();
            DrillingOutHookLoadCatch hookLoadCatch = JSONObject.toJavaObject(JSONObject.parseObject(JSON.toJSONString(redisUtil.get(redisHasKey))), DrillingOutHookLoadCatch.class);
            if (!hookLoadCatch.getIfDetermineWorkingConditions()) {
                Integer recordingSteps = hookLoadCatch.getRecordingSteps();
                List<HookLoadCatch> perColumnData = hookLoadCatch.getPerColumnData();
                List<BigDecimal> collect = perColumnData.stream().map(HookLoadCatch::getHkh).collect(Collectors.toList());
                Optional<BigDecimal> maxHkh = collect.stream().max(BigDecimal::compareTo);
                //上一柱大钩最后的位置
                BigDecimal maxHkhBigDecimal = BigDecimal.ZERO;
                if (perColumnData.size() > 0) {
                    //maxHkh.get();
                    maxHkhBigDecimal = perColumnData.get(perColumnData.size() - 1).getHkh();
                }
                //上一柱大钩位置和当前点大钩位置绝对差值是否大于20米
                BigDecimal hkhWarnValue = BigDecimal.valueOf(18);
                BigDecimal hkhSubtract = maxHkhBigDecimal.subtract(nowHkh).abs();
                int hkhCom = hkhSubtract.compareTo(hkhWarnValue);
                log.info("大钩位置差值是否大于{}米:上一柱大钩最后位置:{},当前点位置:{},位置差值:{}", hkhWarnValue, maxHkhBigDecimal, nowHkh, hkhSubtract);
                if (hkhCom >= 0) {
                    //大钩位置到达最低位置
                    if (recordingSteps == 1) {
                        //判断是否为初次记录上提/下方钻具数据
                        if (perColumnData.size() > 2) {
                            hookLoadCatch.setRecordingSteps(2);
                        } else {
                            hookLoadCatch.setPerColumnData(new ArrayList<>());
                        }
                        hookLoadCatch.setNowColumnData(new ArrayList<>());
                    } else {
                        //如果不是初次记录则将当前记录数据放入上一柱记录数据,清空当前柱记录数据
                        List<HookLoadCatch> perColumnData1 = hookLoadCatch.getPerColumnData();
                        perColumnData1 = hookLoadCatch.getNowColumnData();
                        hookLoadCatch.setPerColumnData(perColumnData1);
                        hookLoadCatch.setNowColumnData(new ArrayList<>());
                    }


                    if (hookLoadCatch.getPerColumnData().size() > 0) {
                        //判断一柱工况
                        String workingCondition = determineWorkingConditions(hookLoadCatch.getPerColumnData());
                        //移除记录柱不在当前工况中的动作数据
                        List<HookLoadCatch> wellDrillings = removeNonOperatingConditionAction(hookLoadCatch.getPerColumnData(), workingCondition);
                        hookLoadCatch.setPerColumnData(wellDrillings);
                        //计算上一柱钩载参考值
                        BigDecimal wohMaximum = getWohMaximum(hookLoadCatch.getPerColumnData(), workingCondition);
                        hookLoadCatch.setAlarmReferenceValue(wohMaximum);

                        hookLoadCatch.setWorkingCondition(workingCondition);
                        hookLoadCatch.setIfDetermineWorkingConditions(true);
                    }
                    hookLoadCatch = initializeMobileDrillingToolsData(hookLoadCatch);
                    redisUtil.set(redisHasKey, hookLoadCatch);
                }
            }
        }
    }


    /**
     * 初始化活动钻具记录数据
     *
     * @param hookLoadCatch
     */
    public DrillingOutHookLoadCatch initializeMobileDrillingToolsData(DrillingOutHookLoadCatch hookLoadCatch) {
        hookLoadCatch.setMobileDrillingToolsStep(0);
        hookLoadCatch.setReferenceMobileDrillingToolsStartData(null);
        hookLoadCatch.setMobileDrillingToolsStartData(null);
        hookLoadCatch.setFirstPointWohFluctuatedValue(BigDecimal.valueOf(0));
        hookLoadCatch.setReferenceMobileDrillingToolsData(new ArrayList<>());
        hookLoadCatch.setMobileDrillingToolsData(new ArrayList<>());
        return hookLoadCatch;
    }

    /**
     * 移除当前柱不属于该工况参考值的数据
     *
     * @param list             已记录柱数据
     * @param workingCondition 工况
     */
    public List<HookLoadCatch> removeNonOperatingConditionAction(List<HookLoadCatch> list, String workingCondition) {
        List<HookLoadCatch> resList = new ArrayList<>();
        //根据工况保留工况下的动作数据
        if (workingCondition.equals(WorkingConditionEnum.TripOut.getValue())) {
            //起钻筛选起钻动作数据
            resList = list.stream().filter(entity -> CommonUtils.diagnosisWarnAction(TripOutActionEnum.class, entity.getMotion()))
                    .collect(Collectors.toList());
            if (resList.size() > 2) {
                resList.remove(0);
                //默认移除最后一个点位数据
                resList.remove(resList.size() - 1);
            }
            return resList;
        } else {

            //下钻保留下钻动作数据
            resList = list.stream().filter(entity -> CommonUtils.diagnosisWarnAction(DrillDownActionEnum.class, entity.getMotion()))
                    .collect(Collectors.toList());
            if (resList.size() > 2) {
                resList.remove(0);
                //默认移除最后一个点位数据
                resList.remove(resList.size() - 1);
            }
            return resList;
        }
    }

    /**
     * 持续起下钻预警判断规则
     *
     * @param nowPoint      当前点
     * @param hookLoadCatch 缓存数据
     * @param rodWeight     一柱杆重
     * @param warnValue     报警阈值
     * @param activityOne     报警阈值
     * @param activityTwo     报警阈值
     * @return 缓存数据
     */
    public DrillingOutHookLoadCatch continuousTrippingCalculationRules(WellDrilling nowPoint, DrillingOutHookLoadCatch hookLoadCatch, BigDecimal rodWeight,
                                                                       BigDecimal warnValue ,BigDecimal activityOne,BigDecimal activityTwo) {
        int res = -1;
        BigDecimal nowPointWoh = nowPoint.getWoh();
        BigDecimal alarmReferenceValue = hookLoadCatch.getAlarmReferenceValue();
        String workingCondition = hookLoadCatch.getWorkingCondition();
        String nowPointMotion = nowPoint.getMotion();
        //判断是否为活动钻具
        Boolean isMobileDrillingTools = judgeIsMobileDrillingTools(workingCondition, nowPointMotion);
        if (!isMobileDrillingTools) {
            if (WorkingConditionEnum.TripOut.getValue().equals(workingCondition)) {
                //持续起钻
                //当（当前点钩载+1柱钻杆重量）-前一柱钩载最大值
                BigDecimal rodAdd = nowPointWoh.add(rodWeight);
                BigDecimal wohSubtract = rodAdd.subtract(alarmReferenceValue);
                res = wohSubtract.compareTo(warnValue);
                // Boolean isWarn=res >= 0;
                if (res >= 0) {
                    log.info("当前点为预警点:时间:{};钩载:{}", nowPoint.getTime(), nowPoint.getWoh());
                    hookLoadCatch.setIsWarnNowPoint(true);
                    List<HookLoadCatch> perColumnData = hookLoadCatch.getNowColumnData();
                    List<BigDecimal> wohList = perColumnData.stream().map(HookLoadCatch::getWoh).collect(Collectors.toList());
                    BigDecimal minReferenceWoh = wohList.stream().min(BigDecimal::compareTo).get();
                    BigDecimal maxReferenceWoh = wohList.stream().max(BigDecimal::compareTo).get();
                    hookLoadWarnLogService.saveHookLoadWarnLog(nowPoint, nowPoint.getOperatingMode(), workingCondition, nowPoint.getMotion(), perColumnData.get(0).getTime(), perColumnData.get(perColumnData.size() - 1).getTime(),
                            hookLoadCatch.getAlarmReferenceValue(), maxReferenceWoh, minReferenceWoh, rodWeight, warnValue);
                } else {
                    hookLoadCatch.setIsWarnNowPoint(false);
                }
                log.info("持续起钻计算公式:((当前点钩载){}+(钻杆重量){}={})-(上一柱钩载参考值){}={},报警阈值:{}", nowPointWoh, rodWeight, rodAdd, alarmReferenceValue, wohSubtract, warnValue);
            } else if (WorkingConditionEnum.DrillingIn.getValue().equals(workingCondition)) {
                //下钻
                //（当前点钩载-1柱钻杆重量）-前一柱钩载最小值
                BigDecimal wohSubtract = nowPointWoh.subtract(rodWeight);
                BigDecimal alarmReferenceSubtract = alarmReferenceValue.subtract(wohSubtract);
                res = alarmReferenceSubtract.compareTo(warnValue);
                if (res >= 0) {
                    log.info("当前点为预警点:时间:{};钩载:{}", nowPoint.getTime(), nowPoint.getWoh());
                    List<HookLoadCatch> nowColumnData = hookLoadCatch.getNowColumnData();
                    List<HookLoadCatch> perColumnData = hookLoadCatch.getPerColumnData();
                    BigDecimal nowHkh = nowPoint.getHkh();
                    if(nowColumnData.size()>0){
                        //WellDrilling perPoint = perColumnData.get(perColumnData.size() - 1);
                        List<BigDecimal> collect = nowColumnData.stream().map(HookLoadCatch::getHkh).collect(Collectors.toList());
                        Optional<BigDecimal> maxHkh =  collect.stream().max(BigDecimal::compareTo);
                        BigDecimal perHkh= maxHkh.get();
                      //  BigDecimal perHkh = perPoint.getHkh();
                        BigDecimal subtract = perHkh.subtract(nowHkh.setScale(2,RoundingMode.HALF_UP));
                        int i = subtract.compareTo(BigDecimal.valueOf(25));
                        if(i<0){
                            hookLoadCatch.setIsWarnNowPoint(true);
                            List<BigDecimal> wohList = perColumnData.stream().map(HookLoadCatch::getWoh).collect(Collectors.toList());
                            BigDecimal minReferenceWoh = wohList.stream().min(BigDecimal::compareTo).get();
                            BigDecimal maxReferenceWoh = wohList.stream().max(BigDecimal::compareTo).get();
                            hookLoadWarnLogService.saveHookLoadWarnLog(nowPoint, nowPoint.getOperatingMode(), workingCondition, nowPoint.getMotion(), perColumnData.get(0).getTime(), perColumnData.get(perColumnData.size() - 1).getTime(),
                                    hookLoadCatch.getAlarmReferenceValue(), maxReferenceWoh, minReferenceWoh, rodWeight, warnValue);
                        }else{
                            hookLoadCatch.setIsWarnNowPoint(false);
                        }
                    }else{
                        hookLoadCatch.setIsWarnNowPoint(false);
                    }

                } else {
                    hookLoadCatch.setIsWarnNowPoint(false);
                }
                log.info("计算公式:(上一柱钩载参考值){}-((当前点钩载){}-(钻杆重量){}={})={},报警阈值:{}", alarmReferenceValue, nowPointWoh, rodWeight, wohSubtract, alarmReferenceSubtract, warnValue);
            }
        }
        hookLoadCatch = mobileDrillingTools(hookLoadCatch, nowPoint, isMobileDrillingTools, activityOne,activityTwo);
        return hookLoadCatch;
    }

    /**
     * 判断是否有活动钻具数据处理
     *
     * @param hookLoadCatch
     * @param nowPoint
     * @param isMobileDrillingTools
     * @return
     */
    public DrillingOutHookLoadCatch mobileDrillingTools(DrillingOutHookLoadCatch hookLoadCatch, WellDrilling nowPoint, Boolean isMobileDrillingTools,
                                                        BigDecimal activityOne,BigDecimal activityTwo) {
        HookLoadCatch point = createPoint(nowPoint);
        if (isMobileDrillingTools) {
            hookLoadCatch = recordMobileDrillingToolsData(hookLoadCatch, nowPoint);
        } else {
            //活动钻具记录步骤
            Integer mobileDrillingToolsStep = hookLoadCatch.getMobileDrillingToolsStep();
            List<HookLoadCatch> referenceMobileDrillingToolsData = hookLoadCatch.getReferenceMobileDrillingToolsData();
            List<BigDecimal> referenceWohList = referenceMobileDrillingToolsData.stream().map(HookLoadCatch::getWoh).collect(Collectors.toList());
            switch (mobileDrillingToolsStep) {
                case 0:
                    hookLoadCatch.setReferenceMobileDrillingToolsStartData(point);
                    break;
                case 1:
                    //结束第一次活动钻具参考数据记录
                    hookLoadCatch.setMobileDrillingToolsStep(2);
                    BigDecimal firstPointWohFluctuatedValue = calculateVolatility(referenceWohList);
                    hookLoadCatch.setFirstPointWohFluctuatedValue(firstPointWohFluctuatedValue);
                    activeDrillingToolsWarnCal(nowPoint.getJh(), nowPoint.getOperatingMode(), nowPoint.getMotion(), hookLoadCatch, activityTwo, 1);
                    break;
                case 2:
                    hookLoadCatch.setMobileDrillingToolsStartData(point);
                    break;
                case 3:
                    //结束第二次及以上次数活动钻具数据记录并计算预警
                    List<HookLoadCatch> mobileDrillingToolsData = hookLoadCatch.getMobileDrillingToolsData();
                    List<BigDecimal> wohList = mobileDrillingToolsData.stream().map(HookLoadCatch::getWoh).collect(Collectors.toList());
                    BigDecimal pointWohFluctuatedValue = calculateVolatility(wohList);
                    //
                    activeDrillingToolsWarnCal(nowPoint.getJh(), nowPoint.getOperatingMode(), nowPoint.getMotion(), hookLoadCatch, activityTwo, 2);

                    //
                    Boolean isWarn = mobileDrillingToolsCalculationRules(pointWohFluctuatedValue, hookLoadCatch.getFirstPointWohFluctuatedValue(), activityOne);
                    hookLoadCatch.setMobileDrillingToolsStep(2);
                    if (isWarn) {
                        log.info("活动钻具预警:参考柱数据{},参考柱波动值:{},当前活动钻具数据:{},当前活动钻具波动值:{}",
                                hookLoadCatch.getReferenceMobileDrillingToolsData(), hookLoadCatch.getFirstPointWohFluctuatedValue(),
                                mobileDrillingToolsData, pointWohFluctuatedValue);
                        BigDecimal mobilDrillingToolsMaxWoh = wohList.stream().min(BigDecimal::compareTo).get();
                        BigDecimal mobilDrillingToolsMinWoh = wohList.stream().max(BigDecimal::compareTo).get();
                        BigDecimal minReferenceWoh = referenceWohList.stream().min(BigDecimal::compareTo).get();
                        BigDecimal maxReferenceWoh = referenceWohList.stream().max(BigDecimal::compareTo).get();
                        hookLoadWarnLogService.saveMobilDrillingToolsHookLoadWarnLog(nowPoint.getJh(), nowPoint.getOperatingMode(), hookLoadCatch.getWorkingCondition() + "-" + "活动钻具", nowPoint.getMotion(),
                                mobileDrillingToolsData.get(0).getTime(), mobileDrillingToolsData.get(mobileDrillingToolsData.size() - 1).getTime(), pointWohFluctuatedValue, mobilDrillingToolsMaxWoh, mobilDrillingToolsMinWoh,
                                referenceMobileDrillingToolsData.get(0).getTime(), referenceMobileDrillingToolsData.get(referenceMobileDrillingToolsData.size() - 1).getTime(), hookLoadCatch.getFirstPointWohFluctuatedValue(),
                                maxReferenceWoh, minReferenceWoh, activityOne);
                    }
                    hookLoadCatch.setMobileDrillingToolsData(new ArrayList<>());
                    break;
                default:
                    break;
            }
        }
        return hookLoadCatch;
    }


    /**
     * 插入预警数据
     *
     * @param nowPoint       当前点
     * @param warnCondition  报警点工况动作
     * @param wholeCondition 整体工况
     * @param bitdepth       钻头监测阈值
     * @param hkhRang        大钩变化
     * @param normalWoh      正常柱钩载最值
     * @param normalStartime 正常柱起始时间
     * @param normalEndtime  正常柱结束时间
     * @param columnWeight   钻杆重量
     * @param warnValue      报警阈值
     */
    @SneakyThrows
    public void saveWarnHistory(WellDrilling nowPoint, String warnCondition, String wholeCondition, Float bitdepth, Float hkhRang, Float normalWoh, Date normalStartime, Date normalEndtime, Float columnWeight, Float warnValue) {
        String des = "符合钩载变化异常预警条件(当前点大钩负荷:" + nowPoint.getWoh() + " KN," + "当前点时间" + nowPoint.getTime() +
                "参考柱钩载:" + normalWoh + " KN,参考柱起点时间:" + sdf.format(normalStartime) + ",参考柱结束时间:" + sdf.format(normalEndtime) +
                "钻杆重量:" + columnWeight + " KN,大于预警值:" + warnValue + " KN);";
        WarnHistory history = new WarnHistory();
        history.setJh(nowPoint.getJh());
        history.setWarnHkh(nowPoint.getHkh().floatValue());
        history.setWarnWoh(nowPoint.getWoh().floatValue());
        history.setWarnType("钩载变化异常");
        history.setWarnTime(sdf.parse(nowPoint.getTime()));
        history.setWarnBitdepth(nowPoint.getBitdepth().floatValue());
        history.setWarnCondition(warnCondition);
        history.setWarnDes(des);
        history.setWholeCondition(wholeCondition);
        history.setNormalStartime(normalStartime);
        history.setNormalEndtime(normalEndtime);
        history.setNormalWoh(normalWoh);
        history.setBitdepth(bitdepth);
        history.setColumnWeight(columnWeight);
        history.setWarnValue(warnValue);
        history.setHkhRang(hkhRang);
        warnHistoryService.save(history);
    }

    /**
     * 记录活动钻具数据
     *
     * @param hookLoadCatch 缓存数据
     * @param nowPoint      当前点
     * @return HookLoadCatch 缓存数据
     */
    public DrillingOutHookLoadCatch recordMobileDrillingToolsData(DrillingOutHookLoadCatch hookLoadCatch, WellDrilling nowPoint) {
        HookLoadCatch point = createPoint(nowPoint);
        //活动钻具步骤
        Integer mobileDrillingToolsStep = hookLoadCatch.getMobileDrillingToolsStep();
        //参考数据
        List<HookLoadCatch> referenceMobileDrillingToolsData = hookLoadCatch.getReferenceMobileDrillingToolsData();
        //预警数据
        List<HookLoadCatch> mobileDrillingToolsData = hookLoadCatch.getMobileDrillingToolsData();
        HookLoadCatch referenceMobileDrillingToolsStartData = hookLoadCatch.getReferenceMobileDrillingToolsStartData();
        //判断样本起点数据是否存在
        if (referenceMobileDrillingToolsStartData != null) {
            switch (mobileDrillingToolsStep) {
                case 0:
                    //未开始
                    List<HookLoadCatch> nowColumnData = hookLoadCatch.getNowColumnData();
                    if (nowColumnData.size() > 0) {
                        HookLoadCatch lastPoint = nowColumnData.get(nowColumnData.size() - 1);
                        String workingCondition = hookLoadCatch.getWorkingCondition();
                        Boolean isMobileDrillingTools = judgeIsMobileDrillingTools(workingCondition, lastPoint.getMotion());
                        if (!isMobileDrillingTools) {
                            hookLoadCatch.setMobileDrillingToolsStep(1);
                            referenceMobileDrillingToolsData.add(referenceMobileDrillingToolsStartData);
                            referenceMobileDrillingToolsData.add(point);
                            hookLoadCatch.setReferenceMobileDrillingToolsData(referenceMobileDrillingToolsData);
                            //   hookLoadCatch.setNowColumnData(new ArrayList<>());
                        }
                    }
                    break;
                case 1:
                    referenceMobileDrillingToolsData.add(point);
                    hookLoadCatch.setReferenceMobileDrillingToolsData(referenceMobileDrillingToolsData);
                    break;
                case 2:
                    hookLoadCatch.setMobileDrillingToolsStep(3);
                    mobileDrillingToolsData.add(hookLoadCatch.getMobileDrillingToolsStartData());
                    mobileDrillingToolsData.add(point);
//                    hookLoadCatch.setNowColumnData(new ArrayList<>());
                    hookLoadCatch.setMobileDrillingToolsData(mobileDrillingToolsData);
                    break;
                case 3:
                    mobileDrillingToolsData.add(point);
                    hookLoadCatch.setMobileDrillingToolsData(mobileDrillingToolsData);
                    break;
                default:
                    break;
            }
        }
        return hookLoadCatch;
    }

    /**
     * 判断是否为活动钻具
     */
    public Boolean judgeIsMobileDrillingTools(String workingCondition, String nowPointMotion) {
        //判断工况是否是起钻工况
        if (workingCondition.equals(WorkingConditionEnum.TripOut.getValue())) {
            //判断当前点动作是否是下放钻具或者边放边循环动作
            return nowPointMotion.equals(TrippingConditionDeterminationEnum.LoweringDrillingTools.getValue()) || nowPointMotion.equals(TrippingConditionDeterminationEnum.LoopWhilePlacing.getValue());
        } else {
            //判断当前点动作是否是上提钻具或者边提边循环动作
            return nowPointMotion.equals(TrippingConditionDeterminationEnum.LiftingDrillingTools.getValue()) || nowPointMotion.equals(TrippingConditionDeterminationEnum.LoopWhileLifting.getValue());
        }
    }

    /**
     * 活动钻具判断规则:当前柱的钩载波动量-这一柱第一次钩载波动量的差值>=50KN
     *
     * @param nowPointWohFluctuatedValue   当前柱的钩载波动量
     * @param firstPointWohFluctuatedValue 这一柱第一次钩载波动量
     * @param warnValue                    预警值
     * @return true:报警/false:不报警
     */
    public Boolean mobileDrillingToolsCalculationRules(BigDecimal nowPointWohFluctuatedValue, BigDecimal firstPointWohFluctuatedValue, BigDecimal warnValue) {
        int res = -1;
        BigDecimal fluctuatedValueSubtract = nowPointWohFluctuatedValue.subtract(firstPointWohFluctuatedValue);
        res = fluctuatedValueSubtract.compareTo(warnValue);
        return res >= 0;
    }

    /**
     * 计算最大值与最小值差值
     *
     * @param list 钩载数组
     * @return 波动值
     */
    public BigDecimal calculateVolatility(List<BigDecimal> list) {
        Optional<BigDecimal> min = list.stream().min(BigDecimal::compareTo);
        Optional<BigDecimal> max = list.stream().max(BigDecimal::compareTo);
        return max.get().subtract(min.get());
    }


    /**
     * 根据钻头位置判断起下钻工况
     *
     * @param list 上提/下方钻具记录数据
     * @return 工况:起钻/下钻
     */
    public String determineWorkingConditions(List<HookLoadCatch> list) {
        String res = "";
        //记录上提/下方钻具动作数据中的钻
        if (list.size() == 0) {
            return res;
        }
        HookLoadCatch oneWellDrilling = list.get(0);
        HookLoadCatch lastWellDrilling = list.get(list.size() - 1);
        BigDecimal oneBitDepth = oneWellDrilling.getBitdepth();
        BigDecimal lastBitDepth = lastWellDrilling.getBitdepth();
        BigDecimal bitDepthSubtract = oneBitDepth.subtract(lastBitDepth);
        int bitDepthRes = bitDepthSubtract.compareTo(BigDecimal.ONE);
        if (bitDepthRes > 0) {
            //起钻
            res = WorkingConditionEnum.TripOut.getValue();
        } else if (bitDepthRes < 0) {
            //下钻
            res = WorkingConditionEnum.DrillingIn.getValue();
        }
        log.info("计算工况:{}", res);
        return res;
    }

    /**
     * 根据当前工况获取钩载最大/最小值
     *
     * @param list             钩载缓存数据
     * @param workingCondition 当前工况
     * @return 钩载最值
     */
    public BigDecimal getWohMaximum(List<HookLoadCatch> list, String workingCondition) {
        BigDecimal res = BigDecimal.ZERO;
        if (list.size() == 0) {
            return res;
        }
        List<BigDecimal> collect = list.stream().map(HookLoadCatch::getWoh).collect(Collectors.toList());
        if (workingCondition.equals(WorkingConditionEnum.TripOut.getValue())) {
            //起钻
            res = collect.stream().max(BigDecimal::compareTo).get();
        } else if (workingCondition.equals(WorkingConditionEnum.DrillingIn.getValue())) {
            //下钻
            res = collect.stream().min(BigDecimal::compareTo).get();
        }
        return res;
    }

    public void activeDrillingToolsWarnCal(String jh, String operatingMode, String motion, DrillingOutHookLoadCatch hookLoadCatch, BigDecimal warnValue, Integer step) {
        List<HookLoadCatch> activeDrillingToolsData = new ArrayList<>();
        if (step == 1) {
            activeDrillingToolsData = hookLoadCatch.getReferenceMobileDrillingToolsData();
        } else {
            activeDrillingToolsData = hookLoadCatch.getMobileDrillingToolsData();
        }
        Integer mobileDrillingToolsStep = hookLoadCatch.getMobileDrillingToolsStep();
        if (mobileDrillingToolsStep > 1) {
            HookLoadWarnLog hookLoadWarnLog = activeDrillingToolsWarn(activeDrillingToolsData);
            BigDecimal bitWoh = hookLoadWarnLog.getHkhWoh();
            int warnCom = bitWoh.compareTo(warnValue);
            log.info("结果值是否大于预警值:{}", warnCom >= 0);
            if (warnCom >= 0) {
                HookLoadWarnLog model = hookLoadWarnLogService.creatNewMobilDrillingToolsHookLoadWarnLog(jh, operatingMode, hookLoadCatch.getWorkingCondition() + "-" + "活动钻具", motion,
                        hookLoadWarnLog.getMobilDrillingToolsWoh(), hookLoadWarnLog.getHkhSub(), activeDrillingToolsData.get(0).getTime(), hookLoadWarnLog.getMobilDrillingToolsMaxWoh(),
                        hookLoadWarnLog.getMobilDrillingToolsMinWoh(), activeDrillingToolsData.get(activeDrillingToolsData.size() - 1).getTime(),
                        hookLoadWarnLog.getHkhStart(), hookLoadWarnLog.getHkhEnd(),
                        hookLoadWarnLog.getHkhWoh(), warnValue);
                log.info(model.getWarnDes());
                hookLoadWarnLogService.insert(model);
            }
        }
    }

    /**
     * 活动钻具数据提取,预警比较
     *
     * @param activeDrillingToolsData
     * @return
     */
    public HookLoadWarnLog activeDrillingToolsWarn(List<HookLoadCatch> activeDrillingToolsData) {
        // DrillingOutHookLoadCatch hookLoadCatch= new DrillingOutHookLoadCatch();
        BigDecimal hkhStartPosition = activeDrillingToolsData.get(0).getHkh();
        BigDecimal hkhEndPosition = activeDrillingToolsData.get(activeDrillingToolsData.size() - 1).getHkh();
        List<BigDecimal> wohList = activeDrillingToolsData.stream().map(HookLoadCatch::getWoh).collect(Collectors.toList());
        BigDecimal wohMax = wohList.stream().max(BigDecimal::compareTo).get();
        BigDecimal wohMin = wohList.stream().min(BigDecimal::compareTo).get();
        return activeDrillingToolsCalculate(hkhStartPosition, hkhEndPosition, wohMax, wohMin);
    }

    /**
     * 活动钻具计算规则：钩载波动值除以钻头位置差值>阈值
     *
     * @param hkhStartPosition 钻头起点位置
     * @param hkhEndPosition   钻头结束位置
     * @param wohMax           钩载最大值
     * @param wohMin           钩载最小值
     */
    public HookLoadWarnLog activeDrillingToolsCalculate(BigDecimal hkhStartPosition, BigDecimal hkhEndPosition, BigDecimal wohMax, BigDecimal wohMin) {
        //钩载波动值除以钻头位置差值>阈值
        //钻头位置差值
        BigDecimal bitSub = hkhStartPosition.subtract(hkhEndPosition).abs();
        //钩载波动值
        BigDecimal wohSub = wohMax.subtract(wohMin).abs();
        BigDecimal res = wohSub.divide(bitSub, 2, RoundingMode.HALF_UP);
        log.info("活动钻具钻头位置差值:起点:{}-结束点:{}={}", hkhStartPosition, hkhEndPosition, bitSub);
        log.info("钩载波动值:最大值:{}-最小值:{}={}", wohMax, wohMin, wohSub);
        log.info("波动值{}➗位置差值:{}={}", wohSub, bitSub, res);
        HookLoadWarnLog hookLoadWarnLog = hookLoadWarnLogService.creatNewMobilDrillingToolsHookLoadWarnLog("", "", "", "",
                wohSub, bitSub, "", wohMax, wohMin, "", hkhStartPosition, hkhEndPosition, res, BigDecimal.ZERO);
        return hookLoadWarnLog;
    }

    public HookLoadCatch createPoint(WellDrilling point){
        HookLoadCatch model=new HookLoadCatch();
        model.setTid(point.getTid());
        model.setJh(point.getJh());
        model.setTime(point.getTime());
        model.setBitdepth(point.getBitdepth());
        model.setWoh(point.getWoh());
        model.setHkh(point.getHkh());
        model.setDepth(point.getDepth());
        return model;
    }

}
