package com.cn.tianji.service.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.cn.tianji.common.CommonUtils;
import com.cn.tianji.common.RedisUtil;
import com.cn.tianji.common.TorqueCatch;
import com.cn.tianji.entity.TorqueWarnLog;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.enums.DrillingConditionDeterminationEnum;
import com.cn.tianji.service.TorqueService;
import com.cn.tianji.service.TorqueWarnLogService;
import com.cn.tianji.service.WellDrillingService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 扭矩预警计算服务层
 * @author Zhao RenJi
 */
@Slf4j
@Service
public class TorqueServiceImpl implements TorqueService {

    @Resource
    private WellDrillingService wellDrillingService;

    @Resource
    private TorqueWarnLogService torqueWarnLogService;
    @Resource
    private RedisUtil redisUtil;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    /**
     *基准样本数据:在钻头位置距离井底0.5米开始记录的5分钟内扭矩变化的数据
     *变化量:
     *（当前点60秒内扭矩变化数据的平均值-样本数据扭矩平均值）/（样本数据扭矩平均值）>5%
     *波动量:
     *（当前点60秒内扭矩变化数据的最大值-最小值）-（基本样本数据的最大值-最小值）/（样本数据的最大值-最小值）>5%
     * 当扭矩预警点连续报警三次后，后面新的报警点再出现时，计算报警点与上一点扭矩波动量是否超过样本数据波动量，如果超过，则报警，如果没超过则不报警
     */
    @Override
    public void torqueCalculate(WellDrilling nowPoint, String redisHasKey,BigDecimal varianceWarnValue,BigDecimal fluctuateWarn){
        String jh = nowPoint.getJh();
        Date nowDate=new Date();
        BigDecimal warnValue = BigDecimal.valueOf(0.05);
        //String redisHasKey = jh + "_torque";
        boolean isHaveCatch = redisUtil.hasKey(redisHasKey);
        String motion = nowPoint.getMotion();
        //判断是否为旋转钻进
        if(motion.equals(DrillingConditionDeterminationEnum.RotaryDrilling.getValue())){
            TorqueCatch torqueCatch = judgeOnWarn(nowPoint, isHaveCatch, redisHasKey,varianceWarnValue,fluctuateWarn);
            if(torqueCatch!=null){
                redisUtil.set(redisHasKey,torqueCatch);
            }
        }else{
            //判断是否存在缓存数据,如果存在则继续判断
            //判断当前工况是否为坐卡
            if(isHaveCatch&&motion.equals(DrillingConditionDeterminationEnum.InSlips.getValue())){
                //移除上一柱缓存数据
                TorqueCatch perTorqueCatch = JSONObject.toJavaObject(JSONObject.parseObject(JSON.toJSONString(redisUtil.get(redisHasKey))), TorqueCatch.class);
                //删除缓存数据
                redisUtil.del(redisHasKey);
                //重新初始化缓存数据,并将上一柱钻头接触井底5分钟的扭矩缓存数据放入缓存数据中
                TorqueCatch  torqueCatch = new TorqueCatch();
                torqueCatch.setPreviousSampleData(perTorqueCatch.getNowInitSampleData());
                torqueCatch.setRecordStep(0);
                redisUtil.set(redisHasKey,torqueCatch);
            }
        }
        Date endDate=new Date();
        long warnTimeDiff = CommonUtils.timeDifference_(nowDate, endDate);
        log.info("计算耗时:{}毫秒",warnTimeDiff);
    }

    /**
     * 扭矩预警计算
     * @param nowPoint 当前点
     * @param isHaveCatch 是否有缓存数据
     */
    public TorqueCatch judgeOnWarn(WellDrilling nowPoint,boolean isHaveCatch,String redisHasKey,BigDecimal varianceWarnValue,BigDecimal fluctuateWarn){
        //判断是否接触井底
        Boolean isOnBottom = judgeOnBottom(nowPoint);
        TorqueCatch torqueCatch =new TorqueCatch();
        //判断是否有缓存数据
        if(isHaveCatch){
            torqueCatch = JSONObject.toJavaObject(JSONObject.parseObject(JSON.toJSONString(redisUtil.get(redisHasKey))), TorqueCatch.class);
            Integer recordStep = torqueCatch.getRecordStep();
            //计算样本数据
            List<WellDrilling> sampleData = torqueCatch.getSampleData();
            //预警计算数据
            List<WellDrilling> catchData = torqueCatch.getCatchData();
            if(recordStep==0){
                //没有缓存数据则开始记录缓存数据
                if(isOnBottom){
                    torqueCatch=initCatchData(torqueCatch,nowPoint);
                }
            }else if(recordStep==1){
                //当前柱初始缓存数据
                List<WellDrilling> nowInitSampleData = torqueCatch.getNowInitSampleData();
                //判断是否结束记录样本数据
                Boolean isEndRecordSampleData = judgeEndRecordSampleData(nowInitSampleData, nowPoint, 5 * 60);
                if(isEndRecordSampleData){
                    sampleData.add(nowPoint);
                    torqueCatch.setSampleData(sampleData);
                    torqueCatch.setSampleData(sampleData);
                    torqueCatch.setRecordStep(2);
                }else{
                    nowInitSampleData.add(nowPoint);
                    sampleData.add(nowPoint);
                    torqueCatch.setNowInitSampleData(nowInitSampleData);
                    torqueCatch.setSampleData(sampleData);
                }
                Boolean isEndRecordSampleData1 = judgeEndRecordSampleData(catchData, nowPoint, 60);
                if(isEndRecordSampleData1){
                    List<WellDrilling> previousSampleData = torqueCatch.getPreviousSampleData();
                    if(previousSampleData!=null){
                        TorqueWarnLog torqueWarnLog = torqueWarn(previousSampleData, catchData,varianceWarnValue,fluctuateWarn);
                        // createModel(nowPoint,catchData,previousSampleData,torqueWarnLog);
                        saveWarnLog(torqueWarnLog,nowPoint,catchData,previousSampleData);
                    }
                    catchData.remove(0);
                }
                catchData.add(nowPoint);
                torqueCatch.setCatchData(catchData);
            }else{
                TorqueWarnLog torqueWarnLog = torqueWarn(sampleData,catchData,varianceWarnValue,fluctuateWarn);
                saveWarnLog(torqueWarnLog,nowPoint,catchData,sampleData);
                sampleData.remove(0);
                sampleData.add(nowPoint);
                catchData.remove(0);
                catchData.add(nowPoint);
                torqueCatch.setSampleData(sampleData);
                torqueCatch.setCatchData(catchData);
            }
        }else{
            //没有缓存数据则开始记录缓存数据
            if(isOnBottom){
                torqueCatch= new TorqueCatch();
                //初始化缓存数据
                torqueCatch=initCatchData(torqueCatch,nowPoint);
            }else{
                torqueCatch=null;
            }
        }
        return torqueCatch;
    }


    /**
     * 新增预警日志数据
     * @param torqueWarnLog
     * @param nowPoint
     * @param catchData
     * @param sampleData
     */
    public void saveWarnLog(TorqueWarnLog torqueWarnLog,WellDrilling nowPoint,List<WellDrilling> catchData,List<WellDrilling> sampleData){
        if("1".equals(torqueWarnLog.getIsMeanWarn())||"1".equals(torqueWarnLog.getIsFluctuatedWarn())){
            TorqueWarnLog model = createModel(nowPoint, catchData, sampleData, torqueWarnLog);
            torqueWarnLogService.saveTorqueWarnLog(model);
        }
    }

    /**
     * 判断是否接触井底
     * @param nowPoint 当前点数据
     */
    public Boolean judgeOnBottom(WellDrilling nowPoint){
        //钻头位置
        BigDecimal bitdepth = nowPoint.getBitdepth();
        //井深
        BigDecimal depth = nowPoint.getDepth();
        //井深钻头差值
        BigDecimal depthSubtract = depth.subtract(bitdepth);
        int i = depthSubtract.compareTo(BigDecimal.valueOf(0));
        return i<=0;
    }

    /**
     * 初始化缓存数据
     * @param torqueCatch 缓存数据
     * @param nowPoint 当前点
     * @return torqueCatch 缓存数据
     */
    public TorqueCatch initCatchData(TorqueCatch torqueCatch,WellDrilling nowPoint){
        torqueCatch.setJh(nowPoint.getJh());
        List<WellDrilling> catchData = new ArrayList<>();
        catchData.add(nowPoint);
        torqueCatch.setNowInitSampleData(catchData);
        torqueCatch.setSampleData(catchData);
        torqueCatch.setCatchData(catchData);
        torqueCatch.setRecordStep(1);
        return torqueCatch;
    }


    /**
     * 判断是否结束样本数据时间
     * @param list 样本数据
     * @param nowPoint 当前点
     * @return true/false
     */
    @SneakyThrows
    public Boolean judgeEndRecordSampleData(List<WellDrilling> list, WellDrilling nowPoint,long endDuration){
        WellDrilling wellDrilling = list.get(0);
        Date startTime = sdf.parse(wellDrilling.getTime());
        Date endTime = sdf.parse(nowPoint.getTime());
        long timeDiff = CommonUtils.timeDifference(startTime, endTime);
        return timeDiff>endDuration;
    }

    /**
     * 扭矩预警
     * @param sampleData 样本数据
     * @param warnData 预警数据
     */
    public TorqueWarnLog torqueWarn(List<WellDrilling> sampleData,List<WellDrilling> warnData,BigDecimal varianceWarnValue,BigDecimal fluctuateWarn){
        List<BigDecimal> torqueCatchData = warnData.stream().map(WellDrilling::getTorque).collect(Collectors.toList());
        List<BigDecimal> torqueSampleData = sampleData.stream().map(WellDrilling::getTorque).collect(Collectors.toList());
        //预警数据扭矩均值
        BigDecimal torqueCatchDataAvg = CommonUtils.calculateAverage(torqueCatchData);
        //预警数据扭矩拨动值
        BigDecimal sampleFluctuatedValueCatchData = CommonUtils.calculateDifference(torqueCatchData);

        //样本数据扭矩均值
        BigDecimal torqueSampleDataAvg = CommonUtils.calculateAverage(torqueSampleData);
        //样本数据扭矩拨动值
        BigDecimal sampleFluctuatedValueSampleData = CommonUtils.calculateDifference(torqueSampleData);
        TorqueWarnLog torqueWarnLog=new TorqueWarnLog();
        //判断是否均值预警
        torqueWarnLog = varianceWarn(torqueSampleDataAvg, torqueCatchDataAvg,varianceWarnValue,torqueWarnLog);
        //判断是否波动值预警
        torqueWarnLog = fluctuateWarn(sampleFluctuatedValueSampleData, sampleFluctuatedValueCatchData,fluctuateWarn,torqueWarnLog);
//        //是否变化量预警
//        if("1".equals(torqueWarnLog.getIsMeanWarn())){
//        log.info("变化量预警,预警点扭矩均值:{};样本扭矩:{}",torqueCatchDataAvg,torqueSampleDataAvg);
//        }
//        //是否波动量阈值
//        if("1".equals(torqueWarnLog.getIsFluctuatedWarn())){
//            log.info("波动量预警,预警点扭矩波动值:{};样本扭矩:{}",torqueCatchDataAvg,sampleFluctuatedValueSampleData);
//        }
        return torqueWarnLog;
    }

    /**
     * 创建扭矩预警实体类数据
     * @param nowPoint 当前点时间
     * @param warnData 预警数据
     * @param sampleData 样本数据
     * @param torqueWarnLog 扭矩预警实体类
     * @return TorqueWarnLog
     */
    @SneakyThrows
    public TorqueWarnLog createModel(WellDrilling nowPoint, List<WellDrilling> warnData, List<WellDrilling> sampleData,TorqueWarnLog torqueWarnLog){
        return torqueWarnLogService.createTorqueWarnLog(nowPoint.getJh(), nowPoint.getOperatingMode(), nowPoint.getMotion(), sdf.parse(warnData.get(0).getTime()), sdf.parse(warnData.get(warnData.size() - 1).getTime()),
                nowPoint.getTorque(), torqueWarnLog.getWarnMeanValue(), torqueWarnLog.getWarnFluctuatedValue(), nowPoint.getBitdepth(), nowPoint.getDepth(), sdf.parse(sampleData.get(0).getTime()), sdf.parse(sampleData.get(sampleData.size() - 1).getTime()),
                torqueWarnLog.getReferenceMeanValue(), torqueWarnLog.getReferenceFluctuatedValue(), torqueWarnLog.getResMeanValue(), torqueWarnLog.getResFluctuatedValue(), torqueWarnLog.getThresholdMeanValue(), torqueWarnLog.getThresholdFluctuatedValue(),
                torqueWarnLog.getIsMeanWarn(),torqueWarnLog.getIsFluctuatedWarn());
    }
    /**
     * 变化量判断:（当前点60秒内扭矩变化数据的平均值-样本数据扭矩平均值）/（样本数据扭矩平均值）>5%
     * @param torqueAvg 样本扭矩平均值
     * @param torqueCatchDataAvg 缓存数据扭矩平均值
     * @param warnValue 预警阈值
     */
    public TorqueWarnLog varianceWarn(BigDecimal torqueAvg, BigDecimal torqueCatchDataAvg, BigDecimal warnValue, TorqueWarnLog torqueWarnLog){
        BigDecimal torqueSubtract = torqueCatchDataAvg.subtract(torqueAvg).abs();
        BigDecimal torqueDivide = torqueSubtract.divide(torqueAvg, 2, RoundingMode.HALF_UP);
        log.info("变化量计算公式:当前点60秒内扭矩变化数据的平均值{}-样本数据扭矩平均值{}={}",torqueCatchDataAvg,torqueAvg,torqueSubtract);
        log.info("变化量计算公式:(当前点60秒内扭矩变化数据的平均值-样本数据扭矩平均值){}/样本数据扭矩平均值{}={}",torqueSubtract,torqueAvg,torqueDivide);
        int res = torqueDivide.compareTo(warnValue);
        //计算结果值
        torqueWarnLog.setResMeanValue(torqueDivide);
        //预警阈值
        torqueWarnLog.setThresholdMeanValue(warnValue);
        //参考扭矩均值
        torqueWarnLog.setReferenceMeanValue(torqueAvg);
        //预警扭矩均值
        torqueWarnLog.setWarnMeanValue(torqueCatchDataAvg);
        if(res>0){
            torqueWarnLog.setIsMeanWarn("1");
        }else{
            torqueWarnLog.setIsMeanWarn("0");
        }
        return torqueWarnLog;
    }

    /**
     * 波动量判断:（当前点60秒内扭矩变化数据的最大值-最小值）-（基本样本数据的最大值-最小值）/（样本数据的最大值-最小值）>5%
     * @param sampleFluctuatedValue 样本波动值
     * @param sampleFluctuatedValueCatchData 缓存波动值
     * @param warnValue 预警阈值
     */
    public TorqueWarnLog fluctuateWarn(BigDecimal sampleFluctuatedValue,BigDecimal sampleFluctuatedValueCatchData,BigDecimal warnValue, TorqueWarnLog torqueWarnLog){
        BigDecimal fluctuatedValueSubtract = sampleFluctuatedValueCatchData.subtract(sampleFluctuatedValue).abs();
        BigDecimal divide = fluctuatedValueSubtract.divide(sampleFluctuatedValue,2, RoundingMode.HALF_UP);
        log.info("波动量计算公式:当前点60秒内扭矩变化数据的波动值{}-样本数据扭矩波动值{}={}",sampleFluctuatedValueCatchData,sampleFluctuatedValue,fluctuatedValueSubtract);
        log.info("波动量计算公式:(当前点60秒内扭矩变化数据的波动值-样本数据扭矩波动值){}/样本数据扭矩波动值{}={}",fluctuatedValueSubtract,sampleFluctuatedValue,divide);
        int res = divide.compareTo(warnValue);
        torqueWarnLog.setResFluctuatedValue(divide);
        torqueWarnLog.setThresholdFluctuatedValue(warnValue);
        torqueWarnLog.setReferenceFluctuatedValue(sampleFluctuatedValue);
        torqueWarnLog.setWarnFluctuatedValue(sampleFluctuatedValueCatchData);
        if(res>0){
            torqueWarnLog.setIsFluctuatedWarn("1");
        }else{
            torqueWarnLog.setIsFluctuatedWarn("0");
        }
        return torqueWarnLog;
    }

}
