package com.cn.tianji.service.impl;

import com.cn.tianji.entity.OperatingConditionThreshold;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.enums.TrippingConditionDeterminationEnum;
import com.cn.tianji.service.CommonConditionDeterminationService;
import com.cn.tianji.service.TrippingConditionDeterminationService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 起下钻工况动作判定
 * @author Zhao RenJi
 */
@Service
@Slf4j
public class TrippingConditionDeterminationServiceImpl implements TrippingConditionDeterminationService {

    @Resource
    private ConditionDeterminationServiceImpl conditionDeterminationService;

    @Resource
    private CommonConditionDeterminationService commonConditionDeterminationService;


    /**
     * 起下钻工况动作判定
     * @param timeSubBig 当前点与上一点时间差
     */
    @Override
    public String trippingConditionDetermination(List<WellDrilling> wellDrillingList, BigDecimal timeSubBig, OperatingConditionThreshold operatingConditionThreshold){
        String res= "";
        //log.info("起下钻排序前的数据:{}",wellDrillingList);
        WellDrilling nowPoint=wellDrillingList.get(wellDrillingList.size()-1);
        WellDrilling perPoint=wellDrillingList.get(0);
        BigDecimal nowWohBig = nowPoint.getWoh();
        log.info("起下钻识别点 时间:{};钩载值:{}",nowPoint.getTime(),nowWohBig);
        //到达地面阈值
        BigDecimal approachingSurfaceThresholdBig=operatingConditionThreshold.getApproachingSurface();
        //坐卡阈值
        BigDecimal wobThresholdBig=operatingConditionThreshold.getWoh();
        List<BigDecimal> bitDepthList = wellDrillingList.stream().map(WellDrilling::getBitdepth).collect(Collectors.toList());
        Boolean isApproachingSurface = conditionDeterminationService.isApproachingSurface(bitDepthList, approachingSurfaceThresholdBig);
        //是否在地面
        if(isApproachingSurface){
            res= TrippingConditionDeterminationEnum.AtSurface.getValue();
        }else{
            //是否坐卡
            Boolean isInSlips = conditionDeterminationService.isInSlips(nowWohBig, wobThresholdBig);
            if(isInSlips){
                res= TrippingConditionDeterminationEnum.InSlips.getValue();
            }else{
                res= publicJudgment(nowPoint,perPoint,timeSubBig,operatingConditionThreshold);
            }
        }
        return res;
    }

    public String publicJudgment(WellDrilling nowPoint, WellDrilling perPoint, BigDecimal timeSubBig,OperatingConditionThreshold operatingConditionThreshold){
        String res= "";
        //判断游车是否静止
        Boolean isBlockldle = commonConditionDeterminationService.publicBlockldle(nowPoint, perPoint, timeSubBig,operatingConditionThreshold.getBlockldle());
        //判断是否停泵及立压为0
        Boolean isPumpAndSpp = commonConditionDeterminationService.publicPumpAndSpp(nowPoint,operatingConditionThreshold.getNoFlow(),operatingConditionThreshold.getPressureZero());
        //判断是否停转
        Boolean publicRotationOff = commonConditionDeterminationService.publicRotationOff(nowPoint,operatingConditionThreshold.getIsRotationOff());
        if(isBlockldle){
            //判断是否停泵及立压为0
            if(isPumpAndSpp){
                //判断是否停转
                if(publicRotationOff){
                    res= TrippingConditionDeterminationEnum.StopAndWait.getValue();
                }else{
                    res= TrippingConditionDeterminationEnum.Idling.getValue();
                }
            }else{
                if(publicRotationOff){
                    res=TrippingConditionDeterminationEnum.MidwayLoop.getValue();
                }else{
                    res= TrippingConditionDeterminationEnum.FixedPointTurntableCycle.getValue();
                }
            }
        }else{
            //判断游车是否上行
            Boolean publicMovingUp = commonConditionDeterminationService.publicMovingUp(nowPoint, perPoint, timeSubBig,
                    operatingConditionThreshold.getBitMovingUp(),operatingConditionThreshold.getBlockMovingUp());
            if(publicMovingUp){
                //判断是否停泵及立压为0
                if(isPumpAndSpp){
                    //判断是否停转
                    if(publicRotationOff){
                        res= TrippingConditionDeterminationEnum.LiftingDrillingTools.getValue();
                    }else{
                        res= TrippingConditionDeterminationEnum.DryInvertedScratch.getValue();
                    }
                }else{
                    if(publicRotationOff){
                        res=TrippingConditionDeterminationEnum.LoopWhileLifting.getValue();
                    }else{
                        res= TrippingConditionDeterminationEnum.ReversesStroke.getValue();
                    }
                }
            }else {
                //判断是否停泵及立压为0
                if(isPumpAndSpp){
                    //判断是否停转
                    if(publicRotationOff){
                        res= TrippingConditionDeterminationEnum.LoweringDrillingTools.getValue();
                    }else{
                        res= TrippingConditionDeterminationEnum.DrysScratch.getValue();
                    }
                }else{
                    if(publicRotationOff){
                        res=TrippingConditionDeterminationEnum.LoopWhilePlacing.getValue();
                    }else{
                        res= TrippingConditionDeterminationEnum.Reaming.getValue();
                    }
                }
            }
        }
        return res;
    }
}
