package com.cn.tianji.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.entity.WellDrillingSICP;
import com.cn.tianji.entity.WellName;
import com.cn.tianji.mapper.WellDrillingSICPMapper;
import com.cn.tianji.service.WellDrillingSICPService;
import com.cn.tianji.service.WellDrillingService;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@DS("yjDataSource")
@Service
@Slf4j
public class WellDrillingSICPServiceImpl implements WellDrillingSICPService {

    @Resource
    private WellDrillingSICPMapper mapper;

    @Resource
    private WellDrillingService wellDrillingService;

    @Override
    public List<WellDrillingSICP> selectListOrcale(String jhTableName, QueryWrapper<WellDrillingSICP> queryWrapper) {
        return mapper.selectListOrcale(jhTableName, queryWrapper);
    }

    @Override
    public String selectMaxTime(String jh) {
        return mapper.selectMaxTime("\""+jh+"\"");
    }

    //筛选不包含在所有井表名中的数组中
    //创建SICP表

    /**
     * 批量创建SICP井表
     */
    public void createTableBatch(List<WellName> list){
        //井表名处理
        list.forEach(item->{
            String wellName = item.getWellName().replace('-', '_').toUpperCase().trim();
            String wellNameLtd= "LTD_"+wellName;
            String wellNameSicp= "SICP_"+wellName;
            item.setWellTableNameLtd(wellNameLtd);
            item.setWellTableNameSicp(wellNameSicp);
        });
        //已有井表数据
        List<String> tablesFromDb = getTablesFromDb();
        //筛选未创建的井表
        list.forEach(item->{
            if(!tablesFromDb.contains(item.getWellTableNameSicp())){
                //根据井表名创建井表
                createTable(item.getWellTableNameSicp(),System.currentTimeMillis());
            }
        });
    }

    @Override
    public List<String> getTablesFromDb() {
        //查询数据库所有井表名
        List<String> jhList = wellDrillingService.tableNames();
        jhList.remove("SYNC_RECORD");
        jhList.remove("WELL_MATH_TIME");
        jhList.remove("WELL_NEW_TABLE");
        jhList.remove("WELL_MATH_UP_TIME");
        jhList.remove("DRILLING_CONDITIONS_LOG");
        return (List<String>) jhList.stream().map(it -> "\"" + it + "\"").collect(Collectors.toList());
    }

    //单表创建
    @Override
    public void createTable(String paramString,long paramLong){
         mapper.createTable(paramString,paramLong);
    }

    @Override
    public  int insertTableBatch(String tableName,  List<WellDrillingSICP> dataList){
        log.info("批量插入数据:表名{};数据量:{}",tableName,dataList.size());
        return mapper.insertTableBatch( "\"" +tableName+ "\"",dataList);
    }
}
