package com.cn.tianji.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cn.tianji.entity.WellDrilling;
import com.cn.tianji.entity.WellDrillingSICP;
import com.cn.tianji.mapper.WellDrillingMapper;
import com.cn.tianji.service.WellDrillingService;
import com.cn.tianji.util.CommonUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

/**
 * @author Zhao RenJi
 */
@Service
public class WellDrillingServiceImpl extends ServiceImpl<WellDrillingMapper, WellDrilling> implements WellDrillingService {

    @Resource
    private WellDrillingMapper mapper;

    @Override
    public List<WellDrilling> selectListOrcale(String jhTableName, QueryWrapper<WellDrilling> queryWrapper) {
        return mapper.selectListOrcale(jhTableName, queryWrapper);
    }

    /**
     * 根据井号名称获得表代码
     *
     * @param AJH
     * @return
     */
    @Override
    public String getTableCode(String AJH) {
        //井号预处理
        String s = AJH.replace('-', '_').toUpperCase();
        return "\"LTD_" + s ;
    }

    @Override
    public void updateByTimeOrcale(String jhTableName, String motion,
                                String operatingMode, QueryWrapper<WellDrilling> queryWrapper){
        mapper.updateByTimeOrcale(jhTableName, motion,operatingMode,queryWrapper);
    }

    @Override
    public  int insertTableBatch(String tableName,List<WellDrilling> dataList){
        return mapper.insertTableBatch(tableName, dataList);
    }

    @Override
    public  List<String> tableNames(){
        return mapper.tableNames();
    }
    @Override
    public String getLastDateByJh( String tableName){
        return mapper.getLastDateByJh(tableName);
    };

    @Override
    public void createTable(String paramString, long paramLong){
       mapper.createTable(paramString,paramLong);
    };


    @Override
    public  List<WellDrilling> sicpToLtdData(List<WellDrillingSICP> sicpList) {
        List<WellDrilling> saveList = new ArrayList<>();
        sicpList.forEach(item -> {
            WellDrilling model = sicpToLtd(item);
            saveList.add(model);
        });
        saveList.sort(Comparator.comparing(WellDrilling::getTime));
        return saveList ;
    }

    @Override
    public WellDrilling sicpToLtd(WellDrillingSICP item) {
        WellDrilling model = new WellDrilling();
        model.setJh(item.getJh());
        model.setTid(item.getTIME());
        model.setTime(item.getTIME());
        model.setId(BigDecimal.valueOf(-999.25));
        try {
            model.setDepth(CommonUtil.convertStringToBigDecimal(item.getDMEA()));
            model.setVdepth(CommonUtil.convertStringToBigDecimal(item.getDVER()));
            model.setLagtime(CommonUtil.convertStringToBigDecimal(item.getTRTT()));
            model.setLagstrokes(CommonUtil.convertStringToBigDecimal(item.getLSTK()));
            model.setBitdepth(CommonUtil.convertStringToBigDecimal(item.getDBTM()));
            model.setKellyin(CommonUtil.convertStringToBigDecimal(item.getKELLYIN()));
            model.setHkh(CommonUtil.convertStringToBigDecimal(item.getBPOS()));
            model.setHks(CommonUtil.convertStringToBigDecimal(item.getBSPD()));
            model.setWoh(CommonUtil.convertStringToBigDecimal(item.getHKLA()));
            model.setRpm(CommonUtil.convertStringToBigDecimal(item.getRPMA()));

            model.setWob(CommonUtil.convertStringToBigDecimal(item.getWOBA()));
            BigDecimal tqa= CommonUtil.convertStringToBigDecimal(item.getTQA());
            if(tqa.compareTo(BigDecimal.valueOf(0.0))>0){
                model.setTorque(tqa);
            }else{
                model.setTorque(CommonUtil.convertStringToBigDecimal(item.getTTQA()));
            }
            model.setTdTorque(CommonUtil.convertStringToBigDecimal(item.getTTQA()));
            model.setSpp(CommonUtil.convertStringToBigDecimal(item.getSPPA()));
            model.setWhp(CommonUtil.convertStringToBigDecimal(item.getCHKP()));
            model.setPump1(CommonUtil.convertStringToBigDecimal(item.getSPM1()));
            model.setPump2(CommonUtil.convertStringToBigDecimal(item.getSPM2()));
            model.setPump3(CommonUtil.convertStringToBigDecimal(item.getSPM3()));
            model.setPumptotal(CommonUtil.convertStringToBigDecimal(item.getTSPM()));

            model.setStrokes1(CommonUtil.convertStringToBigDecimal(item.getSTROKES1()));
            model.setStrokes2(CommonUtil.convertStringToBigDecimal(item.getSTROKES2()));
            model.setStrokes3(CommonUtil.convertStringToBigDecimal(item.getSTROKES3()));
            model.setStrokestotal(CommonUtil.convertStringToBigDecimal(item.getSTKC()));
            model.setVopInst(CommonUtil.convertStringToBigDecimal(item.getVOP_INST()));
            model.setVopMavg(CommonUtil.convertStringToBigDecimal(item.getVOP_MAVG()));
            model.setFlowin(CommonUtil.convertStringToBigDecimal(item.getMFIA()));
            model.setFlowout(CommonUtil.convertStringToBigDecimal(item.getMFOA()));
            model.setFlowoutpercent(CommonUtil.convertStringToBigDecimal(item.getMFOP()));
            model.setMwin(CommonUtil.convertStringToBigDecimal(item.getMDIA()));

            model.setMwout(CommonUtil.convertStringToBigDecimal(item.getMDOA()));
            model.setTempin(CommonUtil.convertStringToBigDecimal(item.getMTIA()));
            model.setTempout(CommonUtil.convertStringToBigDecimal(item.getMTOA()));
            model.setCondin(CommonUtil.convertStringToBigDecimal(item.getMCOA()));
            model.setCondout(CommonUtil.convertStringToBigDecimal(item.getMCIA()));
            model.setPit1(CommonUtil.convertStringToBigDecimal(item.getTV01()));
            model.setPit2(CommonUtil.convertStringToBigDecimal(item.getTV02()));
            model.setPit3(CommonUtil.convertStringToBigDecimal(item.getTV03()));
            model.setPit4(CommonUtil.convertStringToBigDecimal(item.getTV04()));
            model.setPit5(CommonUtil.convertStringToBigDecimal(item.getTV05()));
            model.setPit6(CommonUtil.convertStringToBigDecimal(item.getTV06()));
            model.setPit7(CommonUtil.convertStringToBigDecimal(item.getTV07()));
            model.setPit8(CommonUtil.convertStringToBigDecimal(item.getTV08()));
            model.setPit9(CommonUtil.convertStringToBigDecimal(item.getTV09()));
            model.setPit10(CommonUtil.convertStringToBigDecimal(item.getTV10()));
            model.setPit11(CommonUtil.convertStringToBigDecimal(item.getTV11()));
            model.setPit12(CommonUtil.convertStringToBigDecimal(item.getTV12()));
            model.setPittotal(CommonUtil.convertStringToBigDecimal(item.getTVT()));

            model.setC1(CommonUtil.convertStringToBigDecimal(item.getMETH()));
            model.setC2(CommonUtil.convertStringToBigDecimal(item.getETH()));
            model.setC3(CommonUtil.convertStringToBigDecimal(item.getPRP()));
            model.setIc4(CommonUtil.convertStringToBigDecimal(item.getIBUT()));
            model.setNc4(CommonUtil.convertStringToBigDecimal(item.getNBUT()));
            model.setIc5(CommonUtil.convertStringToBigDecimal(item.getIPEN()));
            model.setNc5(CommonUtil.convertStringToBigDecimal(item.getNPEN()));
            model.setEc5(CommonUtil.convertStringToBigDecimal(item.getEPEN()));
            model.setIc6(CommonUtil.convertStringToBigDecimal(item.getIHEX()));
            model.setNc6(CommonUtil.convertStringToBigDecimal(item.getNHEX()));

            model.setStat(BigDecimal.valueOf(-999.25));
            model.setLagdepth(BigDecimal.valueOf(-999.25));
            model.setBitoff(BigDecimal.valueOf(-999.25));
            model.setWohCalc(BigDecimal.valueOf(-999.25));
            model.setFloatWeight(BigDecimal.valueOf(-999.25));
            model.setRopInst(BigDecimal.valueOf(-999.25));
            model.setRopMavg(BigDecimal.valueOf(-999.25));
            model.setEcd(BigDecimal.valueOf(-999.25));
            model.setTriptandvol(BigDecimal.valueOf(-999.25));
            model.setActpittotal(BigDecimal.valueOf(-999.25));
            model.setGainlossvol(BigDecimal.valueOf(-999.25));
            model.setTg(BigDecimal.valueOf(-999.25));
            model.setCo2(BigDecimal.valueOf(-999.25));
            model.setH2(BigDecimal.valueOf(-999.25));
            model.setH2s1(BigDecimal.valueOf(-999.25));
            model.setH2s2(BigDecimal.valueOf(-999.25));
            model.setH2s3(BigDecimal.valueOf(-999.25));
            model.setH2s4(BigDecimal.valueOf(-999.25));
            model.setH2s5(BigDecimal.valueOf(-999.25));
            model.setH2s6(BigDecimal.valueOf(-999.25));
            model.setH2s7(BigDecimal.valueOf(-999.25));
            model.setH2s8(BigDecimal.valueOf(-999.25));

            model.setStringlen(BigDecimal.valueOf(-999.25));
            model.setCurpipe(BigDecimal.valueOf(-999.25));
            model.setBitno(BigDecimal.valueOf(-999.25));
            model.setBitsize(BigDecimal.valueOf(-999.25));
            model.setBittype(BigDecimal.valueOf(-999.25));
            model.setBitrun(BigDecimal.valueOf(-999.25));
            model.setBittime(BigDecimal.valueOf(-999.25));
            model.setBitdrilltime(BigDecimal.valueOf(-999.25));
            model.setBitcost(BigDecimal.valueOf(-999.25));
            model.setTotaldays(BigDecimal.valueOf(-999.25));

            model.setStandsdone(BigDecimal.valueOf(-999.25));
            model.setStandstogo(BigDecimal.valueOf(-999.25));
            model.setTripoverpull(BigDecimal.valueOf(-999.25));
            model.setDc(BigDecimal.valueOf(-999.25));
            model.setSigma(BigDecimal.valueOf(-999.25));
            model.setFree95(BigDecimal.valueOf(-999.25));
            model.setFree96(BigDecimal.valueOf(-999.25));
            model.setFree97(BigDecimal.valueOf(-999.25));
            model.setFree98(BigDecimal.valueOf(-999.25));
            model.setFree99(BigDecimal.valueOf(-999.25));
            model.setFree100(BigDecimal.valueOf(-999.25));
            model.setFree101(BigDecimal.valueOf(-999.25));
            model.setFree102(BigDecimal.valueOf(-999.25));
            model.setFree103(BigDecimal.valueOf(-999.25));
            model.setFree104(BigDecimal.valueOf(-999.25));
            model.setFree105(BigDecimal.valueOf(-999.25));
            model.setFree106(BigDecimal.valueOf(-999.25));
            model.setFree107(BigDecimal.valueOf(-999.25));
            model.setFree108(BigDecimal.valueOf(-999.25));
            model.setFree109(BigDecimal.valueOf(-999.25));
            model.setFree110(BigDecimal.valueOf(-999.25));
            model.setFree111(BigDecimal.valueOf(-999.25));
            model.setFree112(BigDecimal.valueOf(-999.25));
            model.setFree113(BigDecimal.valueOf(-999.25));
            model.setFree114(BigDecimal.valueOf(-999.25));
            model.setFree115(BigDecimal.valueOf(-999.25));
            model.setFree116(BigDecimal.valueOf(-999.25));
            model.setFree117(BigDecimal.valueOf(-999.25));
            model.setFree118(BigDecimal.valueOf(-999.25));
            model.setFree119(BigDecimal.valueOf(-999.25));
            model.setFree120(BigDecimal.valueOf(-999.25));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return model;
    }
}
